@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  * Copyright (C) 1996-2015, International Business Machines Corporation and    *
@@ -10,7 +10,7 @@ package com.ibm.icu.impl;
 
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.Arrays;
+import java.util.Iterator;
 import java.util.Locale;
 import java.util.regex.Pattern;
 
@@ -18,6 +18,7 @@ import com.ibm.icu.lang.UCharacter;
 import com.ibm.icu.text.Replaceable;
 import com.ibm.icu.text.UTF16;
 import com.ibm.icu.text.UnicodeMatcher;
+import com.ibm.icu.util.ICUUncheckedIOException;
 
 public final class Utility {
 
@@ -182,15 +183,6 @@ public final class Utility {
         return a == b;
     }
 
-    /**
-     * Convenience utility. Does null checks on objects, then calls equals.
-     */
-    public final static boolean objectEquals(Object a, Object b) {
-        return a == null ?
-                b == null ? true : false :
-                    b == null ? false : a.equals(b);
-    }
-
     /**
      * Convenience utility. Does null checks on objects, then calls compare.
      */
@@ -1085,7 +1077,7 @@ public final class Utility {
     public static String[] split(String s, char divider) {
         int last = 0;
         int i;
-        ArrayList<String> output = new ArrayList<String>();
+        ArrayList<String> output = new ArrayList<>();
         for (i = 0; i < s.length(); ++i) {
             if (s.charAt(i) == divider) {
                 output.add(s.substring(last,i));
@@ -1809,46 +1801,6 @@ public final class Utility {
         return buffer.toString();
     }
 
-    /**
-     * This implementation is equivalent to Java 7+ Objects#equals(Object a, Object b)
-     *
-     * @param a an object
-     * @param b an object to be compared with a for equality
-     * @return true if the arguments are equal to each other and false otherwise
-     */
-    public static boolean equals(Object a, Object b) {
-        return (a == b)
-                || (a != null && b != null && a.equals(b));
-    }
-
-    /**
-     * This implementation is equivalent to Java 7+ Objects#hash(Object... values)
-     * @param values the values to be hashed
-     * @return a hash value of the sequence of input values
-     */
-    public static int hash(Object... values) {
-        return Arrays.hashCode(values);
-    }
-
-    /**
-     * This implementation is equivalent to Java 7+ Objects#hashCode(Object o)
-     * @param o an object
-     * @return a hash value of a non-null argument and 0 for null argument
-     */
-    public static int hashCode(Object o) {
-        return o == null ? 0 : o.hashCode();
-    }
-
-    /**
-     * This implementation is equivalent to Java 7+ Objects#toString(Object o)
-     * @param o an object
-     * @return the result of calling toStirng for a non-null argument and "null" for a
-     * null argument
-     */
-    public static String toString(Object o) {
-        return o == null ? "null" : o.toString();
-    }
-
     /**
      * This implementation is equivalent to Java 8+ Math#addExact(int, int)
      * @param x the first value
@@ -1895,4 +1847,44 @@ public final class Utility {
         }
         return hash;
     }
+
+    /**
+     * Appends a CharSequence to an Appendable, converting IOException to ICUUncheckedIOException.
+     */
+    public static <A extends Appendable> A appendTo(CharSequence string, A appendable) {
+        try {
+            appendable.append(string);
+            return appendable;
+        } catch (IOException e) {
+            throw new ICUUncheckedIOException(e);
+        }
+    }
+
+    /**
+     * Java 8+ String#join(CharSequence, Iterable<? extends CharSequence>) compatible method for Java 7 env.
+     * @param delimiter the delimiter that separates each element
+     * @param elements the elements to join together.
+     * @return a new String that is composed of the elements separated by the delimiter
+     * @throws NullPointerException If delimiter or elements is null
+     */
+    public static String joinStrings(CharSequence delimiter, Iterable<? extends CharSequence> elements) {
+        if (delimiter == null || elements == null) {
+            throw new NullPointerException("Delimiter or elements is null");
+        }
+        StringBuilder buf = new StringBuilder();
+        Iterator<? extends CharSequence> itr = elements.iterator();
+        boolean isFirstElem = true;
+        while (itr.hasNext()) {
+            CharSequence element = itr.next();
+            if (element != null) {
+                if (!isFirstElem) {
+                    buf.append(delimiter);
+                } else {
+                    isFirstElem = false;
+                }
+                buf.append(element);
+            }
+        }
+        return buf.toString();
+    }
 }
