@@ -1,5 +1,5 @@
 // © 2017 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 package com.ibm.icu.impl.number;
 
 import java.util.EnumMap;
@@ -12,6 +12,7 @@ import com.ibm.icu.impl.ICUResourceBundle;
 import com.ibm.icu.impl.SimpleFormatterImpl;
 import com.ibm.icu.impl.StandardPlural;
 import com.ibm.icu.impl.UResource;
+import com.ibm.icu.impl.number.Modifier.Signum;
 import com.ibm.icu.number.NumberFormatter.UnitWidth;
 import com.ibm.icu.text.NumberFormat;
 import com.ibm.icu.text.PluralRules;
@@ -21,11 +22,12 @@ import com.ibm.icu.util.MeasureUnit;
 import com.ibm.icu.util.ULocale;
 import com.ibm.icu.util.UResourceBundle;
 
-public class LongNameHandler implements MicroPropsGenerator {
+public class LongNameHandler
+    implements MicroPropsGenerator, ModifierStore, LongNameMultiplexer.ParentlessMicroPropsGenerator {
 
     private static final int DNAM_INDEX = StandardPlural.COUNT;
     private static final int PER_INDEX = StandardPlural.COUNT + 1;
-    private static final int ARRAY_LENGTH = StandardPlural.COUNT + 2;
+    static final int ARRAY_LENGTH = StandardPlural.COUNT + 2;
 
     private static int getIndex(String pluralKeyword) {
         // pluralKeyword can also be "dnam" or "per"
@@ -38,7 +40,7 @@ public class LongNameHandler implements MicroPropsGenerator {
         }
     }
 
-    private static String getWithPlural(String[] strings, StandardPlural plural) {
+    static String getWithPlural(String[] strings, StandardPlural plural) {
         String result = strings[plural.ordinal()];
         if (result == null) {
             result = strings[StandardPlural.OTHER.ordinal()];
@@ -78,7 +80,7 @@ public class LongNameHandler implements MicroPropsGenerator {
 
     // NOTE: outArray MUST have at least ARRAY_LENGTH entries. No bounds checking is performed.
 
-    private static void getMeasureData(
+    static void getMeasureData(
             ULocale locale,
             MeasureUnit unit,
             UnitWidth width,
@@ -97,7 +99,15 @@ public class LongNameHandler implements MicroPropsGenerator {
         key.append("/");
         key.append(unit.getType());
         key.append("/");
-        key.append(unit.getSubtype());
+
+        // Map duration-year-person, duration-week-person, etc. to duration-year, duration-week, ...
+        // TODO(ICU-20400): Get duration-*-person data properly with aliases.
+        if (unit.getSubtype() != null && unit.getSubtype().endsWith("-person")) {
+            key.append(unit.getSubtype(), 0, unit.getSubtype().length() - 7);
+        } else {
+            key.append(unit.getSubtype());
+        }
+
         try {
             resource.getAllItemsWithFallback(key.toString(), sink);
         } catch (MissingResourceException e) {
@@ -175,12 +185,29 @@ public class LongNameHandler implements MicroPropsGenerator {
         String[] simpleFormats = new String[ARRAY_LENGTH];
         getCurrencyLongNameData(locale, currency, simpleFormats);
         // TODO(ICU4J): Reduce the number of object creations here?
-        Map<StandardPlural, SimpleModifier> modifiers = new EnumMap<StandardPlural, SimpleModifier>(
+        Map<StandardPlural, SimpleModifier> modifiers = new EnumMap<>(
                 StandardPlural.class);
-        simpleFormatsToModifiers(simpleFormats, null, modifiers);
-        return new LongNameHandler(modifiers, rules, parent);
+        LongNameHandler result = new LongNameHandler(modifiers, rules, parent);
+        result.simpleFormatsToModifiers(simpleFormats, NumberFormat.Field.CURRENCY);
+        return result;
     }
 
+    /**
+     * Construct a localized LongNameHandler for the specified MeasureUnit.
+     * <p>
+     * Compound units can be constructed via `unit` and `perUnit`. Both of these
+     * must then be built-in units.
+     * <p>
+     * Mixed units are not supported, use MixedUnitLongNameHandler.forMeasureUnit.
+     *
+     * @param locale The desired locale.
+     * @param unit The measure unit to construct a LongNameHandler for. If
+     *     `perUnit` is also defined, `unit` must not be a mixed unit.
+     * @param perUnit If `unit` is a mixed unit, `perUnit` must be null.
+     * @param width Specifies the desired unit rendering.
+     * @param rules Plural rules.
+     * @param parent Plural rules.
+     */
     public static LongNameHandler forMeasureUnit(
             ULocale locale,
             MeasureUnit unit,
@@ -190,8 +217,8 @@ public class LongNameHandler implements MicroPropsGenerator {
             MicroPropsGenerator parent) {
         if (perUnit != null) {
             // Compound unit: first try to simplify (e.g., meters per second is its own unit).
-            MeasureUnit simplified = MeasureUnit.resolveUnitPerUnit(unit, perUnit);
-            if (simplified != null) {
+            MeasureUnit simplified = unit.product(perUnit.reciprocal());
+            if (simplified.getType() != null) {
                 unit = simplified;
             } else {
                 // No simplified form is available.
@@ -199,14 +226,20 @@ public class LongNameHandler implements MicroPropsGenerator {
             }
         }
 
+        if (unit.getType() == null) {
+            // TODO(ICU-20941): Unsanctioned unit. Not yet fully supported.
+            throw new UnsupportedOperationException("Unsanctioned unit, not yet supported: " +
+                                                    unit.getIdentifier());
+        }
+
         String[] simpleFormats = new String[ARRAY_LENGTH];
         getMeasureData(locale, unit, width, simpleFormats);
-        // TODO: What field to use for units?
         // TODO(ICU4J): Reduce the number of object creations here?
-        Map<StandardPlural, SimpleModifier> modifiers = new EnumMap<StandardPlural, SimpleModifier>(
+        Map<StandardPlural, SimpleModifier> modifiers = new EnumMap<>(
                 StandardPlural.class);
-        simpleFormatsToModifiers(simpleFormats, null, modifiers);
-        return new LongNameHandler(modifiers, rules, parent);
+        LongNameHandler result = new LongNameHandler(modifiers, rules, parent);
+        result.simpleFormatsToModifiers(simpleFormats, NumberFormat.Field.MEASURE_UNIT);
+        return result;
     }
 
     private static LongNameHandler forCompoundUnit(
@@ -216,6 +249,13 @@ public class LongNameHandler implements MicroPropsGenerator {
             UnitWidth width,
             PluralRules rules,
             MicroPropsGenerator parent) {
+        if (unit.getType() == null || perUnit.getType() == null) {
+            // TODO(ICU-20941): Unsanctioned unit. Not yet fully supported. Set an
+            // error code.
+            throw new UnsupportedOperationException(
+                "Unsanctioned units, not yet supported: " + unit.getIdentifier() + "/" +
+                perUnit.getIdentifier());
+        }
         String[] primaryData = new String[ARRAY_LENGTH];
         getMeasureData(locale, unit, width, primaryData);
         String[] secondaryData = new String[ARRAY_LENGTH];
@@ -231,36 +271,40 @@ public class LongNameHandler implements MicroPropsGenerator {
             String compiled = SimpleFormatterImpl
                     .compileToStringMinMaxArguments(rawPerUnitFormat, sb, 2, 2);
             String secondaryFormat = getWithPlural(secondaryData, StandardPlural.ONE);
+
+            // Some "one" pattern may not contain "{0}". For example in "ar" or "ne" locale.
             String secondaryCompiled = SimpleFormatterImpl
-                    .compileToStringMinMaxArguments(secondaryFormat, sb, 1, 1);
+                    .compileToStringMinMaxArguments(secondaryFormat, sb, 0, 1);
             String secondaryString = SimpleFormatterImpl.getTextWithNoArguments(secondaryCompiled)
                     .trim();
             perUnitFormat = SimpleFormatterImpl.formatCompiledPattern(compiled, "{0}", secondaryString);
         }
-        // TODO: What field to use for units?
-        Map<StandardPlural, SimpleModifier> modifiers = new EnumMap<StandardPlural, SimpleModifier>(
+        Map<StandardPlural, SimpleModifier> modifiers = new EnumMap<>(
                 StandardPlural.class);
-        multiSimpleFormatsToModifiers(primaryData, perUnitFormat, null, modifiers);
-        return new LongNameHandler(modifiers, rules, parent);
+        LongNameHandler result = new LongNameHandler(modifiers, rules, parent);
+        result.multiSimpleFormatsToModifiers(primaryData, perUnitFormat, NumberFormat.Field.MEASURE_UNIT);
+        return result;
     }
 
-    private static void simpleFormatsToModifiers(
+    private void simpleFormatsToModifiers(
             String[] simpleFormats,
-            NumberFormat.Field field,
-            Map<StandardPlural, SimpleModifier> output) {
+            NumberFormat.Field field) {
         StringBuilder sb = new StringBuilder();
         for (StandardPlural plural : StandardPlural.VALUES) {
             String simpleFormat = getWithPlural(simpleFormats, plural);
             String compiled = SimpleFormatterImpl.compileToStringMinMaxArguments(simpleFormat, sb, 0, 1);
-            output.put(plural, new SimpleModifier(compiled, field, false));
+            Modifier.Parameters parameters = new Modifier.Parameters();
+            parameters.obj = this;
+            parameters.signum = null;// Signum ignored
+            parameters.plural = plural;
+            modifiers.put(plural, new SimpleModifier(compiled, field, false, parameters));
         }
     }
 
-    private static void multiSimpleFormatsToModifiers(
+    private void multiSimpleFormatsToModifiers(
             String[] leadFormats,
             String trailFormat,
-            NumberFormat.Field field,
-            Map<StandardPlural, SimpleModifier> output) {
+            NumberFormat.Field field) {
         StringBuilder sb = new StringBuilder();
         String trailCompiled = SimpleFormatterImpl.compileToStringMinMaxArguments(trailFormat, sb, 1, 1);
         for (StandardPlural plural : StandardPlural.VALUES) {
@@ -268,17 +312,39 @@ public class LongNameHandler implements MicroPropsGenerator {
             String compoundFormat = SimpleFormatterImpl.formatCompiledPattern(trailCompiled, leadFormat);
             String compoundCompiled = SimpleFormatterImpl
                     .compileToStringMinMaxArguments(compoundFormat, sb, 0, 1);
-            output.put(plural, new SimpleModifier(compoundCompiled, field, false));
+            Modifier.Parameters parameters = new Modifier.Parameters();
+            parameters.obj = this;
+            parameters.signum = null; // Signum ignored
+            parameters.plural = plural;
+            modifiers.put(plural, new SimpleModifier(compoundCompiled, field, false, parameters));
         }
     }
 
     @Override
     public MicroProps processQuantity(DecimalQuantity quantity) {
         MicroProps micros = parent.processQuantity(quantity);
-        // TODO: Avoid the copy here?
-        DecimalQuantity copy = quantity.createCopy();
-        micros.rounder.apply(copy);
-        micros.modOuter = modifiers.get(copy.getStandardPlural(rules));
+        StandardPlural pluralForm = RoundingUtils.getPluralSafe(micros.rounder, rules, quantity);
+        micros.modOuter = modifiers.get(pluralForm);
         return micros;
     }
+
+    /**
+     * Produces a plural-appropriate Modifier for a unit: `quantity` is taken as
+     * the final smallest unit, while the larger unit values must be provided
+     * via `micros.mixedMeasures`.
+     *
+     * Does not call parent.processQuantity, so cannot get a MicroProps instance
+     * that way. Instead, the instance is passed in as a parameter.
+     */
+    public MicroProps processQuantityWithMicros(DecimalQuantity quantity, MicroProps micros) {
+        StandardPlural pluralForm = RoundingUtils.getPluralSafe(micros.rounder, rules, quantity);
+        micros.modOuter = modifiers.get(pluralForm);
+        return micros;
+    }
+
+    @Override
+    public Modifier getModifier(Signum signum, StandardPlural plural) {
+        // Signum ignored
+        return modifiers.get(plural);
+    }
 }
