@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /**
 *******************************************************************************
 * Copyright (C) 1996-2016, International Business Machines Corporation and
@@ -560,13 +560,20 @@ public interface UProperty
      * @stable ICU 60
      */
     public static final int PREPENDED_CONCATENATION_MARK=63;
+    /**
+     * Binary property Extended_Pictographic.
+     * See http://www.unicode.org/reports/tr51/#Emoji_Properties
+     *
+     * @stable ICU 62
+     */
+    public static final int EXTENDED_PICTOGRAPHIC=64;
 
     /**
      * One more than the last constant for binary Unicode properties.
      * @deprecated ICU 58 The numeric value may change over time, see ICU ticket #12420.
      */
     @Deprecated
-    public static final int BINARY_LIMIT = 64;
+    public static final int BINARY_LIMIT = 65;
 
     /**
      * Enumerated property Bidi_Class.
@@ -746,12 +753,37 @@ public interface UProperty
      */
     public static final int BIDI_PAIRED_BRACKET_TYPE = 0x1015;
 
+    /**
+     * Enumerated property Indic_Positional_Category.
+     * New in Unicode 6.0 as provisional property Indic_Matra_Category;
+     * renamed and changed to informative in Unicode 8.0.
+     * See http://www.unicode.org/reports/tr44/#IndicPositionalCategory.txt
+     * @stable ICU 63
+     */
+    public static final int INDIC_POSITIONAL_CATEGORY = 0x1016;
+
+    /**
+     * Enumerated property Indic_Syllabic_Category.
+     * New in Unicode 6.0 as provisional; informative since Unicode 8.0.
+     * See http://www.unicode.org/reports/tr44/#IndicSyllabicCategory.txt
+     * @stable ICU 63
+     */
+    public static final int INDIC_SYLLABIC_CATEGORY = 0x1017;
+
+    /**
+     * Enumerated property Vertical_Orientation.
+     * Used for UAX #50 Unicode Vertical Text Layout (https://www.unicode.org/reports/tr50/).
+     * New as a UCD property in Unicode 10.0.
+     * @stable ICU 63
+     */
+    public static final int VERTICAL_ORIENTATION = 0x1018;
+
     /**
      * One more than the last constant for enumerated/integer Unicode properties.
      * @deprecated ICU 58 The numeric value may change over time, see ICU ticket #12420.
      */
     @Deprecated
-    public static final int INT_LIMIT = 0x1016;
+    public static final int INT_LIMIT = 0x1019;
 
     /**
      * Bitmask property General_Category_Mask.
