@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /**
  *******************************************************************************
  * Copyright (C) 2001-2016 International Business Machines Corporation and
@@ -1013,6 +1013,42 @@ public final class UScript {
      */
     public static final int ZANABAZAR_SQUARE = 177; /* Zanb */
 
+    /**
+     * ISO 15924 script code
+     * @stable ICU 62
+     */
+    public static final int DOGRA = 178; /* Dogr */
+    /** @stable ICU 62 */
+    public static final int GUNJALA_GONDI = 179; /* Gong */
+    /** @stable ICU 62 */
+    public static final int MAKASAR = 180; /* Maka */
+    /** @stable ICU 62 */
+    public static final int MEDEFAIDRIN = 181; /* Medf */
+    /** @stable ICU 62 */
+    public static final int HANIFI_ROHINGYA = 182; /* Rohg */
+    /** @stable ICU 62 */
+    public static final int SOGDIAN = 183; /* Sogd */
+    /** @stable ICU 62 */
+    public static final int OLD_SOGDIAN = 184; /* Sogo */
+
+    /** @stable ICU 64 */
+    public static final int ELYMAIC = 185; /* Elym */
+    /** @stable ICU 64 */
+    public static final int NYIAKENG_PUACHUE_HMONG = 186; /* Hmnp */
+    /** @stable ICU 64 */
+    public static final int NANDINAGARI = 187; /* Nand */
+    /** @stable ICU 64 */
+    public static final int WANCHO = 188; /* Wcho */
+
+    /** @stable ICU 66 */
+    public static final int CHORASMIAN = 189; /* Chrs */
+    /** @stable ICU 66 */
+    public static final int DIVES_AKURU = 190; /* Diak */
+    /** @stable ICU 66 */
+    public static final int KHITAN_SMALL_SCRIPT = 191; /* Kits */
+    /** @stable ICU 66 */
+    public static final int YEZIDI = 192; /* Yezi */
+
     /**
      * One more than the highest normal UScript code.
      * The highest value is available via UCharacter.getIntPropertyMaxValue(UProperty.SCRIPT).
@@ -1020,7 +1056,7 @@ public final class UScript {
      * @deprecated ICU 58 The numeric value may change over time, see ICU ticket #12420.
      */
     @Deprecated
-    public static final int CODE_LIMIT   = 178;
+    public static final int CODE_LIMIT   = 193;
 
     private static int[] getCodesFromLocale(ULocale locale) {
         // Multi-script languages, equivalent to the LocaleScript data
@@ -1144,14 +1180,15 @@ public final class UScript {
     public static final int getScript(int codepoint){
         if (codepoint >= UCharacter.MIN_VALUE & codepoint <= UCharacter.MAX_VALUE) {
             int scriptX=UCharacterProperty.INSTANCE.getAdditional(codepoint, 0)&UCharacterProperty.SCRIPT_X_MASK;
+            int codeOrIndex=UCharacterProperty.mergeScriptCodeOrIndex(scriptX);
             if(scriptX<UCharacterProperty.SCRIPT_X_WITH_COMMON) {
-                return scriptX;
+                return codeOrIndex;
             } else if(scriptX<UCharacterProperty.SCRIPT_X_WITH_INHERITED) {
                 return UScript.COMMON;
             } else if(scriptX<UCharacterProperty.SCRIPT_X_WITH_OTHER) {
                 return UScript.INHERITED;
             } else {
-                return UCharacterProperty.INSTANCE.m_scriptExtensions_[scriptX&UCharacterProperty.SCRIPT_MASK_];
+                return UCharacterProperty.INSTANCE.m_scriptExtensions_[codeOrIndex];
             }
         }else{
             throw new IllegalArgumentException(Integer.toString(codepoint));
@@ -1173,12 +1210,13 @@ public final class UScript {
      */
     public static final boolean hasScript(int c, int sc) {
         int scriptX=UCharacterProperty.INSTANCE.getAdditional(c, 0)&UCharacterProperty.SCRIPT_X_MASK;
+        int codeOrIndex=UCharacterProperty.mergeScriptCodeOrIndex(scriptX);
         if(scriptX<UCharacterProperty.SCRIPT_X_WITH_COMMON) {
-            return sc==scriptX;
+            return sc==codeOrIndex;
         }
 
         char[] scriptExtensions=UCharacterProperty.INSTANCE.m_scriptExtensions_;
-        int scx=scriptX&UCharacterProperty.SCRIPT_MASK_;  // index into scriptExtensions
+        int scx=codeOrIndex;  // index into scriptExtensions
         if(scriptX>=UCharacterProperty.SCRIPT_X_WITH_OTHER) {
             scx=scriptExtensions[scx+1];
         }
@@ -1222,13 +1260,14 @@ public final class UScript {
     public static final int getScriptExtensions(int c, BitSet set) {
         set.clear();
         int scriptX=UCharacterProperty.INSTANCE.getAdditional(c, 0)&UCharacterProperty.SCRIPT_X_MASK;
+        int codeOrIndex=UCharacterProperty.mergeScriptCodeOrIndex(scriptX);
         if(scriptX<UCharacterProperty.SCRIPT_X_WITH_COMMON) {
-            set.set(scriptX);
-            return scriptX;
+            set.set(codeOrIndex);
+            return codeOrIndex;
         }
 
         char[] scriptExtensions=UCharacterProperty.INSTANCE.m_scriptExtensions_;
-        int scx=scriptX&UCharacterProperty.SCRIPT_MASK_;  // index into scriptExtensions
+        int scx=codeOrIndex;  // index into scriptExtensions
         if(scriptX>=UCharacterProperty.SCRIPT_X_WITH_OTHER) {
             scx=scriptExtensions[scx+1];
         }
@@ -1326,7 +1365,7 @@ public final class UScript {
             0x0EA5 | RECOMMENDED | LB_LETTERS,  // Laoo
             0x004C | RECOMMENDED | CASED,  // Latn
             0x0D15 | RECOMMENDED,  // Mlym
-            0x1826 | LIMITED_USE,  // Mong
+            0x1826 | EXCLUSION,  // Mong
             0x1000 | RECOMMENDED | LB_LETTERS,  // Mymr
             0x168F | EXCLUSION,  // Ogam
             0x10300 | EXCLUSION,  // Ital
@@ -1477,6 +1516,21 @@ public final class UScript {
             0x11D10 | EXCLUSION,  // Gonm
             0x11A5C | EXCLUSION,  // Soyo
             0x11A0B | EXCLUSION,  // Zanb
+            0x1180B | EXCLUSION,  // Dogr
+            0x11D71 | LIMITED_USE,  // Gong
+            0x11EE5 | EXCLUSION,  // Maka
+            0x16E40 | EXCLUSION | CASED,  // Medf
+            0x10D12 | LIMITED_USE | RTL,  // Rohg
+            0x10F42 | EXCLUSION | RTL,  // Sogd
+            0x10F19 | EXCLUSION | RTL,  // Sogo
+            0x10FF1 | EXCLUSION | RTL,  // Elym
+            0x1E108 | LIMITED_USE,  // Hmnp
+            0x119CE | EXCLUSION,  // Nand
+            0x1E2E1 | LIMITED_USE,  // Wcho
+            0x10FBF | EXCLUSION | RTL,  // Chrs
+            0x1190C | EXCLUSION,  // Diak
+            0x18C65 | EXCLUSION | LB_LETTERS,  // Kits
+            0x10E88 | EXCLUSION | RTL,  // Yezi
             // End copy-paste from parsescriptmetadata.py
         };
 
