@@ -1,18 +1,18 @@
 // © 2017 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 package com.ibm.icu.number;
 
 import java.math.BigDecimal;
 import java.math.MathContext;
 
 import com.ibm.icu.impl.number.AffixPatternProvider;
-import com.ibm.icu.impl.number.CurrencyPluralInfoAffixProvider;
 import com.ibm.icu.impl.number.CustomSymbolCurrency;
 import com.ibm.icu.impl.number.DecimalFormatProperties;
 import com.ibm.icu.impl.number.Grouper;
 import com.ibm.icu.impl.number.MacroProps;
 import com.ibm.icu.impl.number.Padder;
 import com.ibm.icu.impl.number.PatternStringParser;
+import com.ibm.icu.impl.number.PatternStringUtils;
 import com.ibm.icu.impl.number.PropertiesAffixPatternProvider;
 import com.ibm.icu.impl.number.RoundingUtils;
 import com.ibm.icu.number.NumberFormatter.DecimalSeparatorDisplay;
@@ -103,12 +103,7 @@ final class NumberPropertyMapper {
         // AFFIXES //
         /////////////
 
-        AffixPatternProvider affixProvider;
-        if (properties.getCurrencyPluralInfo() == null) {
-            affixProvider = new PropertiesAffixPatternProvider(properties);
-        } else {
-            affixProvider = new CurrencyPluralInfoAffixProvider(properties.getCurrencyPluralInfo(), properties);
-        }
+        AffixPatternProvider affixProvider = PropertiesAffixPatternProvider.forProperties(properties);
         macros.affixProvider = affixProvider;
 
         ///////////
@@ -160,10 +155,8 @@ final class NumberPropertyMapper {
         }
         // Validate min/max int/frac.
         // For backwards compatibility, minimum overrides maximum if the two conflict.
-        // The following logic ensures that there is always a minimum of at least one digit.
         if (minInt == 0 && maxFrac != 0) {
-            // Force a digit after the decimal point.
-            minFrac = minFrac <= 0 ? 1 : minFrac;
+            minFrac = (minFrac < 0 || (minFrac == 0 && maxInt == 0)) ? 1 : minFrac;
             maxFrac = maxFrac < 0 ? -1 : maxFrac < minFrac ? minFrac : maxFrac;
             minInt = 0;
             maxInt = maxInt < 0 ? -1 : maxInt > RoundingUtils.MAX_INT_FRAC_SIG ? -1 : maxInt;
@@ -179,7 +172,11 @@ final class NumberPropertyMapper {
         if (explicitCurrencyUsage) {
             rounding = Precision.constructCurrency(currencyUsage).withCurrency(currency);
         } else if (roundingIncrement != null) {
-            rounding = Precision.constructIncrement(roundingIncrement);
+            if (PatternStringUtils.ignoreRoundingIncrement(roundingIncrement, maxFrac)) {
+                rounding = Precision.constructFraction(minFrac, maxFrac);
+            } else {
+                rounding = Precision.constructIncrement(roundingIncrement);
+            }
         } else if (explicitMinMaxSig) {
             minSig = minSig < 1 ? 1
                     : minSig > RoundingUtils.MAX_INT_FRAC_SIG ? RoundingUtils.MAX_INT_FRAC_SIG : minSig;
@@ -214,7 +211,7 @@ final class NumberPropertyMapper {
         // PADDING //
         /////////////
 
-        if (properties.getFormatWidth() != -1) {
+        if (properties.getFormatWidth() > 0) {
             macros.padder = Padder.forProperties(properties);
         }
 
@@ -263,8 +260,8 @@ final class NumberPropertyMapper {
             // TODO: Overriding here is a bit of a hack. Should this logic go earlier?
             if (macros.precision instanceof FractionPrecision) {
                 // For the purposes of rounding, get the original min/max int/frac, since the local
-                // variables
-                // have been manipulated for display purposes.
+                // variables have been manipulated for display purposes.
+                int maxInt_ = properties.getMaximumIntegerDigits();
                 int minInt_ = properties.getMinimumIntegerDigits();
                 int minFrac_ = properties.getMinimumFractionDigits();
                 int maxFrac_ = properties.getMaximumFractionDigits();
@@ -275,8 +272,15 @@ final class NumberPropertyMapper {
                     // Patterns like "#.##E0" (no zeros in the mantissa), which mean round to maxFrac+1
                     macros.precision = Precision.constructSignificant(1, maxFrac_ + 1).withMode(mathContext);
                 } else {
-                    // All other scientific patterns, which mean round to minInt+maxFrac
-                    macros.precision = Precision.constructSignificant(minInt_ + minFrac_, minInt_ + maxFrac_)
+                    int maxSig_ = minInt_ + maxFrac_;
+                    // Bug #20058: if maxInt_ > minInt_ > 1, then minInt_ should be 1.
+                    if (maxInt_ > minInt_ && minInt_ > 1) {
+                        minInt_ = 1;
+                    }
+                    int minSig_ = minInt_ + minFrac_;
+                    // To avoid regression, maxSig is not reset when minInt_ set to 1.
+                    // TODO: Reset maxSig_ = 1 + minFrac_ to follow the spec.
+                    macros.precision = Precision.constructSignificant(minSig_, maxSig_)
                             .withMode(mathContext);
                 }
             }
