@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  * Copyright (C) 1996-2016, International Business Machines Corporation and
@@ -144,8 +144,7 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
      * @param locale the locale.
      * @param ns the numbering system.
      * @return A DecimalFormatSymbols instance.
-     * @provisional This API might change or be removed in a future release.
-     * @draft ICU 60
+     * @stable ICU 60
      */
     public static DecimalFormatSymbols forNumberingSystem(Locale locale, NumberingSystem ns) {
         return new DecimalFormatSymbols(locale, ns);
@@ -164,8 +163,7 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
      * @param locale the locale.
      * @param ns the numbering system.
      * @return A DecimalFormatSymbols instance.
-     * @provisional This API might change or be removed in a future release.
-     * @draft ICU 60
+     * @stable ICU 60
      */
     public static DecimalFormatSymbols forNumberingSystem(ULocale locale, NumberingSystem ns) {
         return new DecimalFormatSymbols(locale, ns);
@@ -890,9 +888,32 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
         if (currency == null) {
             throw new NullPointerException();
         }
+        if (currency.equals(this.currency)) {
+            return;
+        }
+        CurrencyDisplayInfo displayInfo = CurrencyData.provider.getInstance(ulocale, true);
+        setCurrencyOrNull(currency, displayInfo);
+    }
+
+    private void setCurrencyOrNull(Currency currency, CurrencyDisplayInfo displayInfo) {
         this.currency = currency;
+
+        if (currency == null) {
+            intlCurrencySymbol = "XXX";
+            currencySymbol = "\u00A4"; // 'OX' currency symbol
+            currencyPattern = null;
+            return;
+        }
+
         intlCurrencySymbol = currency.getCurrencyCode();
-        currencySymbol = currency.getSymbol(requestedLocale);
+        currencySymbol = currency.getSymbol(ulocale);
+
+        CurrencyFormatInfo formatInfo = displayInfo.getFormatInfo(currency.getCurrencyCode());
+        if (formatInfo != null) {
+            setMonetaryDecimalSeparatorString(formatInfo.monetaryDecimalSeparator);
+            setMonetaryGroupingSeparatorString(formatInfo.monetaryGroupingSeparator);
+            currencyPattern = formatInfo.currencyPattern;
+        }
     }
 
     /**
@@ -1006,11 +1027,12 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
     }
 
     /**
-    }
      * Internal API for NumberFormat
      * @return String currency pattern string
+     * @deprecated This API is for ICU internal use only
      */
-    String getCurrencyPattern() {
+    @Deprecated
+    public String getCurrencyPattern() {
         return currencyPattern;
     }
 
@@ -1158,8 +1180,10 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
             throw new IllegalArgumentException("unknown currency spacing: " + itemType);
         }
         if (beforeCurrency) {
+            currencySpcBeforeSym = currencySpcBeforeSym.clone();
             currencySpcBeforeSym[itemType] = pattern;
         } else {
+            currencySpcAfterSym = currencySpcAfterSym.clone();
             currencySpcAfterSym[itemType] = pattern;
         }
     }
@@ -1272,7 +1296,6 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
     private static final String[] SYMBOL_KEYS = {
             "decimal",
             "group",
-            "list",
             "percentSign",
             "minusSign",
             "plusSign",
@@ -1310,7 +1333,6 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
     private static final String[] SYMBOL_DEFAULTS = new String[] {
             String.valueOf(DEF_DECIMAL_SEPARATOR),  // decimal
             String.valueOf(DEF_GROUPING_SEPARATOR), // group
-            ";", // list
             String.valueOf(DEF_PERCENT),    // percentSign
             String.valueOf(DEF_MINUS_SIGN), // minusSign
             String.valueOf(DEF_PLUS_SIGN),  // plusSign
@@ -1381,50 +1403,27 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
         // Copy data from the numberElements map into instance fields
         setDecimalSeparatorString(numberElements[0]);
         setGroupingSeparatorString(numberElements[1]);
-
-        // See CLDR #9781
-        // assert numberElements[2].length() == 1;
-        patternSeparator = numberElements[2].charAt(0);
-
-        setPercentString(numberElements[3]);
-        setMinusSignString(numberElements[4]);
-        setPlusSignString(numberElements[5]);
-        setExponentSeparator(numberElements[6]);
-        setPerMillString(numberElements[7]);
-        setInfinity(numberElements[8]);
-        setNaN(numberElements[9]);
-        setMonetaryDecimalSeparatorString(numberElements[10]);
-        setMonetaryGroupingSeparatorString(numberElements[11]);
-        setExponentMultiplicationSign(numberElements[12]);
+        // #11897: pattern separator is always ';', not from data
+        patternSeparator = ';';
+        setPercentString(numberElements[2]);
+        setMinusSignString(numberElements[3]);
+        setPlusSignString(numberElements[4]);
+        setExponentSeparator(numberElements[5]);
+        setPerMillString(numberElements[6]);
+        setInfinity(numberElements[7]);
+        setNaN(numberElements[8]);
+        setMonetaryDecimalSeparatorString(numberElements[9]);
+        setMonetaryGroupingSeparatorString(numberElements[10]);
+        setExponentMultiplicationSign(numberElements[11]);
 
         digit = '#';  // Localized pattern character no longer in CLDR
         padEscape = '*';
         sigDigit  = '@';
 
+        CurrencyDisplayInfo displayInfo = CurrencyData.provider.getInstance(ulocale, true);
+        initSpacingInfo(displayInfo.getSpacingInfo());
 
-        CurrencyDisplayInfo info = CurrencyData.provider.getInstance(locale, true);
-
-        // Obtain currency data from the currency API.  This is strictly
-        // for backward compatibility; we don't use DecimalFormatSymbols
-        // for currency data anymore.
-        currency = Currency.getInstance(locale);
-        if (currency != null) {
-            intlCurrencySymbol = currency.getCurrencyCode();
-            currencySymbol = currency.getName(locale, Currency.SYMBOL_NAME, null);
-            CurrencyFormatInfo fmtInfo = info.getFormatInfo(intlCurrencySymbol);
-            if (fmtInfo != null) {
-                currencyPattern = fmtInfo.currencyPattern;
-                setMonetaryDecimalSeparatorString(fmtInfo.monetaryDecimalSeparator);
-                setMonetaryGroupingSeparatorString(fmtInfo.monetaryGroupingSeparator);
-            }
-        } else {
-            intlCurrencySymbol = "XXX";
-            currencySymbol = "\u00A4"; // 'OX' currency symbol
-        }
-
-
-        // Get currency spacing data.
-        initSpacingInfo(info.getSpacingInfo());
+        setCurrencyOrNull(Currency.getInstance(ulocale), displayInfo);
     }
 
     private static CacheData loadData(ULocale locale) {
@@ -1488,11 +1487,11 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
 
         // If monetary decimal or grouping were not explicitly set, then set them to be the same as
         // their non-monetary counterparts.
-        if (numberElements[10] == null) {
-            numberElements[10] = numberElements[0];
+        if (numberElements[9] == null) {
+            numberElements[9] = numberElements[0];
         }
-        if (numberElements[11] == null) {
-            numberElements[11] = numberElements[1];
+        if (numberElements[10] == null) {
+            numberElements[10] = numberElements[1];
         }
 
         return new CacheData(validLocale, digits, numberElements);
@@ -1654,13 +1653,13 @@ public class DecimalFormatSymbols implements Cloneable, Serializable {
      * represents a sequence of ten code points in order.
      *
      * <p>If the value stored here is positive, it means that the code point stored in this value
-     * corresponds to the digitStrings array, and zeroCodePoint can be used instead of the
+     * corresponds to the digitStrings array, and codePointZero can be used instead of the
      * digitStrings array for the purposes of efficient formatting; if -1, then digitStrings does
      * *not* contain a sequence of code points, and it must be used directly.
      *
-     * <p>It is assumed that zeroCodePoint always shadows the value in digitStrings. zeroCodePoint
+     * <p>It is assumed that codePointZero always shadows the value in digitStrings. codePointZero
      * should never be set directly; rather, it should be updated only when digitStrings mutates.
-     * That is, the flow of information is digitStrings -> zeroCodePoint, not the other way.
+     * That is, the flow of information is digitStrings -> codePointZero, not the other way.
      */
     private transient int codePointZero;
 
