@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  **********************************************************************
  * Copyright (c) 2004-2016, International Business Machines
@@ -23,7 +23,6 @@ import java.text.FieldPosition;
 import java.text.ParsePosition;
 import java.util.Arrays;
 import java.util.Collection;
-import java.util.Date;
 import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
@@ -31,12 +30,18 @@ import java.util.MissingResourceException;
 import java.util.concurrent.ConcurrentHashMap;
 
 import com.ibm.icu.impl.DontCareFieldPosition;
+import com.ibm.icu.impl.FormattedStringBuilder;
+import com.ibm.icu.impl.FormattedValueStringBuilderImpl;
 import com.ibm.icu.impl.ICUData;
 import com.ibm.icu.impl.ICUResourceBundle;
 import com.ibm.icu.impl.SimpleCache;
 import com.ibm.icu.impl.SimpleFormatterImpl;
+import com.ibm.icu.impl.Utility;
+import com.ibm.icu.impl.number.DecimalQuantity;
+import com.ibm.icu.impl.number.DecimalQuantity_DualStorageBCD;
 import com.ibm.icu.impl.number.LongNameHandler;
-import com.ibm.icu.number.FormattedNumber;
+import com.ibm.icu.impl.number.RoundingUtils;
+import com.ibm.icu.number.IntegerWidth;
 import com.ibm.icu.number.LocalizedNumberFormatter;
 import com.ibm.icu.number.NumberFormatter;
 import com.ibm.icu.number.NumberFormatter.UnitWidth;
@@ -46,7 +51,6 @@ import com.ibm.icu.util.Currency;
 import com.ibm.icu.util.ICUUncheckedIOException;
 import com.ibm.icu.util.Measure;
 import com.ibm.icu.util.MeasureUnit;
-import com.ibm.icu.util.TimeZone;
 import com.ibm.icu.util.ULocale;
 import com.ibm.icu.util.ULocale.Category;
 import com.ibm.icu.util.UResourceBundle;
@@ -126,9 +130,9 @@ public class MeasureFormat extends UFormat {
 
     private final transient LocalizedNumberFormatter numberFormatter;
 
-    private static final SimpleCache<ULocale, NumericFormatters> localeToNumericDurationFormatters = new SimpleCache<ULocale, NumericFormatters>();
+    private static final SimpleCache<ULocale, NumericFormatters> localeToNumericDurationFormatters = new SimpleCache<>();
 
-    private static final Map<MeasureUnit, Integer> hmsTo012 = new HashMap<MeasureUnit, Integer>();
+    private static final Map<MeasureUnit, Integer> hmsTo012 = new HashMap<>();
 
     static {
         hmsTo012.put(MeasureUnit.HOUR, 0);
@@ -155,21 +159,21 @@ public class MeasureFormat extends UFormat {
          *
          * @stable ICU 53
          */
-        WIDE(ListFormatter.Style.DURATION, UnitWidth.FULL_NAME, UnitWidth.FULL_NAME),
+        WIDE(ListFormatter.Style.UNIT, UnitWidth.FULL_NAME, UnitWidth.FULL_NAME),
 
         /**
          * Abbreviate when possible.
          *
          * @stable ICU 53
          */
-        SHORT(ListFormatter.Style.DURATION_SHORT, UnitWidth.SHORT, UnitWidth.ISO_CODE),
+        SHORT(ListFormatter.Style.UNIT_SHORT, UnitWidth.SHORT, UnitWidth.ISO_CODE),
 
         /**
          * Brief. Use only a symbol for the unit when possible.
          *
          * @stable ICU 53
          */
-        NARROW(ListFormatter.Style.DURATION_NARROW, UnitWidth.NARROW, UnitWidth.SHORT),
+        NARROW(ListFormatter.Style.UNIT_NARROW, UnitWidth.NARROW, UnitWidth.SHORT),
 
         /**
          * Identical to NARROW except when formatMeasures is called with an hour and minute; minute and
@@ -178,7 +182,7 @@ public class MeasureFormat extends UFormat {
          *
          * @stable ICU 53
          */
-        NUMERIC(ListFormatter.Style.DURATION_NARROW, UnitWidth.NARROW, UnitWidth.SHORT),
+        NUMERIC(ListFormatter.Style.UNIT_NARROW, UnitWidth.NARROW, UnitWidth.SHORT),
 
         /**
          * The default format width for getCurrencyFormat(), which is to show the symbol for currency
@@ -188,7 +192,7 @@ public class MeasureFormat extends UFormat {
          * @deprecated ICU 61 This API is ICU internal only.
          */
         @Deprecated
-        DEFAULT_CURRENCY(ListFormatter.Style.DURATION, UnitWidth.FULL_NAME, UnitWidth.SHORT);
+        DEFAULT_CURRENCY(ListFormatter.Style.UNIT, UnitWidth.FULL_NAME, UnitWidth.SHORT);
 
         private final ListFormatter.Style listFormatterStyle;
 
@@ -319,9 +323,10 @@ public class MeasureFormat extends UFormat {
         } else if (obj instanceof Measure[]) {
             formatMeasuresInternal(toAppendTo, fpos, (Measure[]) obj);
         } else if (obj instanceof Measure) {
-            FormattedNumber result = formatMeasure((Measure) obj);
-            result.populateFieldPosition(fpos); // No offset: toAppendTo.length() is considered below
-            result.appendTo(toAppendTo);
+            FormattedStringBuilder result = formatMeasure((Measure) obj);
+            // No offset: toAppendTo.length() is considered below
+            FormattedValueStringBuilderImpl.nextFieldPosition(result, fpos);
+            Utility.appendTo(result, toAppendTo);
         } else {
             throw new IllegalArgumentException(obj.toString());
         }
@@ -385,11 +390,13 @@ public class MeasureFormat extends UFormat {
             MeasureUnit perUnit,
             StringBuilder appendTo,
             FieldPosition pos) {
-        FormattedNumber result = getUnitFormatterFromCache(NUMBER_FORMATTER_STANDARD,
-                measure.getUnit(),
-                perUnit).format(measure.getNumber());
-        DecimalFormat.fieldPositionHelper(result, pos, appendTo.length());
-        result.appendTo(appendTo);
+        DecimalQuantity dq = new DecimalQuantity_DualStorageBCD(measure.getNumber());
+        FormattedStringBuilder string = new FormattedStringBuilder();
+        getUnitFormatterFromCache(
+            NUMBER_FORMATTER_STANDARD, measure.getUnit(), perUnit
+        ).formatImpl(dq, string);
+        DecimalFormat.fieldPositionHelper(dq, string, pos, appendTo.length());
+        Utility.appendTo(string, appendTo);
         return appendTo;
     }
 
@@ -432,9 +439,9 @@ public class MeasureFormat extends UFormat {
             return;
         }
         if (measures.length == 1) {
-            FormattedNumber result = formatMeasure(measures[0]);
-            result.populateFieldPosition(fieldPosition);
-            result.appendTo(appendTo);
+            FormattedStringBuilder result = formatMeasure(measures[0]);
+            FormattedValueStringBuilderImpl.nextFieldPosition(result, fieldPosition);
+            Utility.appendTo(result, appendTo);
             return;
         }
 
@@ -463,7 +470,7 @@ public class MeasureFormat extends UFormat {
                 results[i] = formatMeasureInteger(measures[i]).toString();
             }
         }
-        FormattedListBuilder builder = listFormatter.format(Arrays.asList(results), -1);
+        FormattedListBuilder builder = listFormatter.formatImpl(Arrays.asList(results), false);
         builder.appendTo(appendTo);
     }
 
@@ -486,7 +493,7 @@ public class MeasureFormat extends UFormat {
      * Two MeasureFormats, a and b, are equal if and only if they have the same formatWidth, locale, and
      * equal number formats.
      *
-     * @stable ICU 53
+     * @stable ICU 3.0
      */
     @Override
     public final boolean equals(Object other) {
@@ -506,7 +513,7 @@ public class MeasureFormat extends UFormat {
     /**
      * {@inheritDoc}
      *
-     * @stable ICU 53
+     * @stable ICU 3.0
      */
     @Override
     public final int hashCode() {
@@ -520,6 +527,9 @@ public class MeasureFormat extends UFormat {
      * @stable ICU 53
      */
     public MeasureFormat.FormatWidth getWidth() {
+        if (formatWidth == MeasureFormat.FormatWidth.DEFAULT_CURRENCY) {
+            return MeasureFormat.FormatWidth.WIDE;
+        }
         return formatWidth;
     }
 
@@ -651,28 +661,28 @@ public class MeasureFormat extends UFormat {
     }
 
     static class NumericFormatters {
-        private DateFormat hourMinute;
-        private DateFormat minuteSecond;
-        private DateFormat hourMinuteSecond;
+        private String hourMinute;
+        private String minuteSecond;
+        private String hourMinuteSecond;
 
         public NumericFormatters(
-                DateFormat hourMinute,
-                DateFormat minuteSecond,
-                DateFormat hourMinuteSecond) {
+                String hourMinute,
+                String minuteSecond,
+                String hourMinuteSecond) {
             this.hourMinute = hourMinute;
             this.minuteSecond = minuteSecond;
             this.hourMinuteSecond = hourMinuteSecond;
         }
 
-        public DateFormat getHourMinute() {
+        public String getHourMinute() {
             return hourMinute;
         }
 
-        public DateFormat getMinuteSecond() {
+        public String getMinuteSecond() {
             return minuteSecond;
         }
 
-        public DateFormat getHourMinuteSecond() {
+        public String getHourMinuteSecond() {
             return hourMinuteSecond;
         }
     }
@@ -738,7 +748,8 @@ public class MeasureFormat extends UFormat {
         } else {
             assert type == NUMBER_FORMATTER_INTEGER;
             formatter = getNumberFormatter().unit(unit).perUnit(perUnit).unitWidth(formatWidth.unitWidth)
-                    .rounding(Precision.integer().withMode(RoundingMode.DOWN));
+                    .precision(Precision.integer().withMode(
+                            RoundingUtils.mathContextUnlimited(RoundingMode.DOWN)));
         }
         formatter3 = formatter2;
         formatter2 = formatter1;
@@ -763,20 +774,26 @@ public class MeasureFormat extends UFormat {
 
     /// END NUMBER FORMATTER CACHING MACHINERY ///
 
-    private FormattedNumber formatMeasure(Measure measure) {
+    private FormattedStringBuilder formatMeasure(Measure measure) {
         MeasureUnit unit = measure.getUnit();
+        DecimalQuantity dq = new DecimalQuantity_DualStorageBCD(measure.getNumber());
+        FormattedStringBuilder string = new FormattedStringBuilder();
         if (unit instanceof Currency) {
-            return getUnitFormatterFromCache(NUMBER_FORMATTER_CURRENCY, unit, null)
-                    .format(measure.getNumber());
+            getUnitFormatterFromCache(NUMBER_FORMATTER_CURRENCY, unit, null)
+                    .formatImpl(dq, string);
         } else {
-            return getUnitFormatterFromCache(NUMBER_FORMATTER_STANDARD, unit, null)
-                    .format(measure.getNumber());
+            getUnitFormatterFromCache(NUMBER_FORMATTER_STANDARD, unit, null)
+                    .formatImpl(dq, string);
         }
+        return string;
     }
 
-    private FormattedNumber formatMeasureInteger(Measure measure) {
-        return getUnitFormatterFromCache(NUMBER_FORMATTER_INTEGER, measure.getUnit(), null)
-                .format(measure.getNumber());
+    private FormattedStringBuilder formatMeasureInteger(Measure measure) {
+        DecimalQuantity dq = new DecimalQuantity_DualStorageBCD(measure.getNumber());
+        FormattedStringBuilder string = new FormattedStringBuilder();
+        getUnitFormatterFromCache(NUMBER_FORMATTER_INTEGER, measure.getUnit(), null)
+                .formatImpl(dq, string);
+        return string;
     }
 
     private void formatMeasuresSlowTrack(
@@ -792,38 +809,36 @@ public class MeasureFormat extends UFormat {
 
         int fieldPositionFoundIndex = -1;
         for (int i = 0; i < measures.length; ++i) {
-            FormattedNumber result;
+            FormattedStringBuilder result;
             if (i == measures.length - 1) {
                 result = formatMeasure(measures[i]);
             } else {
                 result = formatMeasureInteger(measures[i]);
             }
             if (fieldPositionFoundIndex == -1) {
-                result.populateFieldPosition(fpos);
+                FormattedValueStringBuilderImpl.nextFieldPosition(result, fpos);
                 if (fpos.getEndIndex() != 0) {
                     fieldPositionFoundIndex = i;
                 }
             }
             results[i] = result.toString();
         }
-        ListFormatter.FormattedListBuilder builder = listFormatter.format(Arrays.asList(results),
-                fieldPositionFoundIndex);
+        ListFormatter.FormattedListBuilder builder = listFormatter.formatImpl(Arrays.asList(results), true);
 
         // Fix up FieldPosition indexes if our field is found.
-        if (builder.getOffset() != -1) {
-            fieldPosition.setBeginIndex(fpos.getBeginIndex() + builder.getOffset());
-            fieldPosition.setEndIndex(fpos.getEndIndex() + builder.getOffset());
+        int offset = builder.getOffset(fieldPositionFoundIndex);
+        if (offset != -1) {
+            fieldPosition.setBeginIndex(fpos.getBeginIndex() + offset);
+            fieldPosition.setEndIndex(fpos.getEndIndex() + offset);
         }
         builder.appendTo(appendTo);
     }
 
     // type is one of "hm", "ms" or "hms"
-    private static DateFormat loadNumericDurationFormat(ICUResourceBundle r, String type) {
+    private static String loadNumericDurationFormat(ICUResourceBundle r, String type) {
         r = r.getWithFallback(String.format("durationUnits/%s", type));
         // We replace 'h' with 'H' because 'h' does not make sense in the context of durations.
-        DateFormat result = new SimpleDateFormat(r.getString().replace("h", "H"));
-        result.setTimeZone(TimeZone.GMT_ZONE);
-        return result;
+        return r.getString().replace("h", "H");
     }
 
     // Returns hours in [0]; minutes in [1]; seconds in [2] out of measures array. If
@@ -856,116 +871,77 @@ public class MeasureFormat extends UFormat {
     // Formats numeric time duration as 5:00:47 or 3:54. In the process, it replaces any null
     // values in hms with 0.
     private void formatNumeric(Number[] hms, Appendable appendable) {
+        String pattern;
 
-        // find the start and end of non-nil values in hms array. We have to know if we
-        // have hour-minute; minute-second; or hour-minute-second.
-        int startIndex = -1;
-        int endIndex = -1;
-        for (int i = 0; i < hms.length; i++) {
-            if (hms[i] != null) {
-                endIndex = i;
-                if (startIndex == -1) {
-                    startIndex = endIndex;
-                }
-            } else {
-                // Replace nil value with 0.
-                hms[i] = Integer.valueOf(0);
+        // All possible combinations: "h", "m", "s", "hm", "hs", "ms", "hms"
+        if (hms[0] != null && hms[2] != null) { // "hms" & "hs" (we add minutes if "hs")
+            pattern = numericFormatters.getHourMinuteSecond();
+            if (hms[1] == null)
+                hms[1] = 0;
+            hms[1] = Math.floor(hms[1].doubleValue());
+            hms[0] = Math.floor(hms[0].doubleValue());
+        } else if (hms[0] != null && hms[1] != null) { // "hm"
+            pattern = numericFormatters.getHourMinute();
+            hms[0] = Math.floor(hms[0].doubleValue());
+        } else if (hms[1] != null && hms[2] != null) { // "ms"
+            pattern = numericFormatters.getMinuteSecond();
+            hms[1] = Math.floor(hms[1].doubleValue());
+        } else { // h m s, handled outside formatNumeric. No value is also an error.
+            throw new IllegalStateException();
+        }
+
+        // We can create it on demand, but all of the patterns (right now) have mm and ss.
+        // So unless it is hours only we will need a 0-padded 2 digits formatter.
+        LocalizedNumberFormatter numberFormatter2 = numberFormatter.integerWidth(IntegerWidth.zeroFillTo(2));
+        FormattedStringBuilder fsb = new FormattedStringBuilder();
+
+        boolean protect = false;
+        for (int i = 0; i < pattern.length(); i++) {
+            char c = pattern.charAt(i);
+
+            // Also set the proper field in this switch
+            // We don't use DateFormat.Field because this is not a date / time, is a duration.
+            Number value = 0;
+            switch (c) {
+                case 'H': value = hms[0]; break;
+                case 'm': value = hms[1]; break;
+                case 's': value = hms[2]; break;
             }
-        }
-        // convert hours, minutes, seconds into milliseconds.
-        long millis = (long) (((Math.floor(hms[0].doubleValue()) * 60.0
-                + Math.floor(hms[1].doubleValue())) * 60.0 + Math.floor(hms[2].doubleValue())) * 1000.0);
-        Date d = new Date(millis);
-        if (startIndex == 0 && endIndex == 2) {
-            // if hour-minute-second
-            formatNumeric(d,
-                    numericFormatters.getHourMinuteSecond(),
-                    DateFormat.Field.SECOND,
-                    hms[endIndex],
-                    appendable);
-        } else if (startIndex == 1 && endIndex == 2) {
-            // if minute-second
-            formatNumeric(d,
-                    numericFormatters.getMinuteSecond(),
-                    DateFormat.Field.SECOND,
-                    hms[endIndex],
-                    appendable);
-        } else if (startIndex == 0 && endIndex == 1) {
-            // if hour-minute
-            formatNumeric(d,
-                    numericFormatters.getHourMinute(),
-                    DateFormat.Field.MINUTE,
-                    hms[endIndex],
-                    appendable);
-        } else {
-            throw new IllegalStateException();
-        }
-    }
 
-    // Formats a duration as 5:00:37 or 23:59.
-    // duration is a particular duration after epoch.
-    // formatter is a hour-minute-second, hour-minute, or minute-second formatter.
-    // smallestField denotes what the smallest field is in duration: either
-    // hour, minute, or second.
-    // smallestAmount is the value of that smallest field. for 5:00:37.3,
-    // smallestAmount is 37.3. This smallest field is formatted with this object's
-    // NumberFormat instead of formatter.
-    // appendTo is where the formatted string is appended.
-    private void formatNumeric(
-            Date duration,
-            DateFormat formatter,
-            DateFormat.Field smallestField,
-            Number smallestAmount,
-            Appendable appendTo) {
-        // Format the smallest amount ahead of time.
-        String smallestAmountFormatted;
-
-        // Format the smallest amount using this object's number format, but keep track
-        // of the integer portion of this formatted amount. We have to replace just the
-        // integer part with the corresponding value from formatting the date. Otherwise
-        // when formatting 0 minutes 9 seconds, we may get "00:9" instead of "00:09"
-        FieldPosition intFieldPosition = new FieldPosition(NumberFormat.INTEGER_FIELD);
-        FormattedNumber result = getNumberFormatter().format(smallestAmount);
-        result.populateFieldPosition(intFieldPosition);
-        smallestAmountFormatted = result.toString();
-        // Give up if there is no integer field.
-        if (intFieldPosition.getBeginIndex() == 0 && intFieldPosition.getEndIndex() == 0) {
-            throw new IllegalStateException();
-        }
-
-        // Format our duration as a date, but keep track of where the smallest field is
-        // so that we can use it to replace the integer portion of the smallest value.
-        // #13606: DateFormat is not thread-safe, but MeasureFormat advertises itself as thread-safe.
-        FieldPosition smallestFieldPosition = new FieldPosition(smallestField);
-        String draft;
-        synchronized (formatter) {
-            draft = formatter.format(duration, new StringBuffer(), smallestFieldPosition).toString();
+            // There is not enough info to add Field(s) for the unit because all we have are plain
+            // text patterns. For example in "21:51" there is no text for something like "hour",
+            // while in something like "21h51" there is ("h"). But we can't really tell...
+            switch (c) {
+                case 'H':
+                case 'm':
+                case 's':
+                    if (protect) {
+                        fsb.appendChar16(c, null);
+                    } else {
+                        if ((i + 1 < pattern.length()) && pattern.charAt(i + 1) == c) { // doubled
+                            fsb.append(numberFormatter2.format(value), null); // TODO: Use proper Field
+                            i++;
+                        } else {
+                            fsb.append(numberFormatter.format(value), null); // TODO: Use proper Field
+                        }
+                    }
+                    break;
+                case '\'':
+                    // '' is escaped apostrophe
+                    if ((i + 1 < pattern.length()) && pattern.charAt(i + 1) == c) {
+                        fsb.appendChar16(c, null);
+                        i++;
+                    } else {
+                        protect = !protect;
+                    }
+                    break;
+                default:
+                    fsb.appendChar16(c, null);
+            }
         }
 
         try {
-            // If we find the smallest field
-            if (smallestFieldPosition.getBeginIndex() != 0 || smallestFieldPosition.getEndIndex() != 0) {
-                // add everything up to the start of the smallest field in duration.
-                appendTo.append(draft, 0, smallestFieldPosition.getBeginIndex());
-
-                // add everything in the smallest field up to the integer portion
-                appendTo.append(smallestAmountFormatted, 0, intFieldPosition.getBeginIndex());
-
-                // Add the smallest field in formatted duration in lieu of the integer portion
-                // of smallest field
-                appendTo.append(draft,
-                        smallestFieldPosition.getBeginIndex(),
-                        smallestFieldPosition.getEndIndex());
-
-                // Add the rest of the smallest field
-                appendTo.append(smallestAmountFormatted,
-                        intFieldPosition.getEndIndex(),
-                        smallestAmountFormatted.length());
-                appendTo.append(draft, smallestFieldPosition.getEndIndex(), draft.length());
-            } else {
-                // As fallback, just use the formatted duration.
-                appendTo.append(draft);
-            }
+            appendable.append(fsb);
         } catch (IOException e) {
             throw new ICUUncheckedIOException(e);
         }
@@ -997,7 +973,7 @@ public class MeasureFormat extends UFormat {
             this.formatWidth = width;
             this.numberFormat = numberFormat;
             this.subClass = subClass;
-            this.keyValues = new HashMap<Object, Object>();
+            this.keyValues = new HashMap<>();
         }
 
         // Must have public constructor, to enable Externalizable
@@ -1070,7 +1046,7 @@ public class MeasureFormat extends UFormat {
         return values[ordinal];
     }
 
-    private static final Map<ULocale, String> localeIdToRangeFormat = new ConcurrentHashMap<ULocale, String>();
+    private static final Map<ULocale, String> localeIdToRangeFormat = new ConcurrentHashMap<>();
 
     /**
      * Return a formatter (compiled SimpleFormatter pattern) for a range, such as "{0}–{1}".
