@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  * Copyright (C) 1996-2016, International Business Machines Corporation and
@@ -1141,8 +1141,9 @@ public abstract class NumberFormat extends UFormat {
 
     /**
      * Overrides equals.
-     * Two NumberFormats are equal if they are of the same class
-     * and the settings (groupingUsed, parseIntegerOnly, maximumIntegerDigits, etc.
+     * Two NumberFormats are equal they are of the same class
+     * and the user-specified values for settings
+     * (groupingUsed, parseIntegerOnly, maximumIntegerDigits, etc.)
      * are equal.
      * @param obj the object to compare against
      * @return true if the object is equal to this.
@@ -1446,8 +1447,7 @@ public abstract class NumberFormat extends UFormat {
         DecimalFormatSymbols symbols = new DecimalFormatSymbols(desiredLocale);
 
         // Here we assume that the locale passed in is in the canonical
-        // form, e.g: pt_PT_@currency=PTE not pt_PT_PREEURO
-        // This style wont work for currency plural format.
+        // form, e.g: pt_PT_@currency=PTE
         // For currency plural format, the pattern is get from
         // the locale (from CurrencyUnitPatterns) without override.
         if (choice == CURRENCYSTYLE || choice == ISOCURRENCYSTYLE || choice == ACCOUNTINGCURRENCYSTYLE
@@ -1960,6 +1960,16 @@ public abstract class NumberFormat extends UFormat {
          */
         public static final Field CURRENCY = new Field("currency");
 
+        /**
+         * @stable ICU 64
+         */
+        public static final Field MEASURE_UNIT = new Field("measure unit");
+
+        /**
+         * @stable ICU 64
+         */
+        public static final Field COMPACT = new Field("compact");
+
         /**
          * Constructs a new instance of NumberFormat.Field with the given field
          * name.
@@ -1998,6 +2008,10 @@ public abstract class NumberFormat extends UFormat {
                 return PERMILLE;
             if (this.getName().equals(SIGN.getName()))
                 return SIGN;
+            if (this.getName().equals(MEASURE_UNIT.getName()))
+                return MEASURE_UNIT;
+            if (this.getName().equals(COMPACT.getName()))
+                return COMPACT;
 
             throw new InvalidObjectException("An invalid object.");
         }
