@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  * Copyright (C) 2007-2016, International Business Machines Corporation and
@@ -15,8 +15,8 @@ import java.text.FieldPosition;
 import java.text.ParsePosition;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Objects;
 
-import com.ibm.icu.impl.Utility;
 import com.ibm.icu.number.FormattedNumber;
 import com.ibm.icu.number.LocalizedNumberFormatter;
 import com.ibm.icu.text.PluralRules.FixedDecimal;
@@ -760,9 +760,15 @@ public class PluralFormat extends UFormat {
 
             String currArg = pattern.substring(partStart.getLimit(), partLimit.getIndex());
             if (scanner != null) {
-                // If lenient parsing is turned ON, we've got some time consuming parsing ahead of us.
-                int[] scannerMatchResult = scanner.findText(source, currArg, startingAt);
-                currMatchIndex = scannerMatchResult[0];
+                // Check if non-lenient rule finds the text before call lenient parsing
+                int tempPos = source.indexOf(currArg, startingAt);
+                if (tempPos >= 0) {
+                    currMatchIndex = tempPos;
+                } else {
+                    // If lenient parsing is turned ON, we've got some time consuming parsing ahead of us.
+                    int[] scannerMatchResult = scanner.findText(source, currArg, startingAt);
+                    currMatchIndex = scannerMatchResult[0];
+                }
             }
             else {
                 currMatchIndex = source.indexOf(currArg, startingAt);
@@ -834,10 +840,10 @@ public class PluralFormat extends UFormat {
         }
         PluralFormat pf = (PluralFormat)rhs;
         return
-            Utility.objectEquals(ulocale, pf.ulocale) &&
-            Utility.objectEquals(pluralRules, pf.pluralRules) &&
-            Utility.objectEquals(msgPattern, pf.msgPattern) &&
-            Utility.objectEquals(numberFormat, pf.numberFormat);
+            Objects.equals(ulocale, pf.ulocale) &&
+            Objects.equals(pluralRules, pf.pluralRules) &&
+            Objects.equals(msgPattern, pf.msgPattern) &&
+            Objects.equals(numberFormat, pf.numberFormat);
     }
 
     /**
