@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  * Copyright (C) 2007-2016, International Business Machines Corporation and
@@ -29,6 +29,11 @@ import java.util.TreeSet;
 import java.util.regex.Pattern;
 
 import com.ibm.icu.impl.PluralRulesLoader;
+import com.ibm.icu.impl.StandardPlural;
+import com.ibm.icu.impl.number.range.StandardPluralRanges;
+import com.ibm.icu.number.FormattedNumber;
+import com.ibm.icu.number.FormattedNumberRange;
+import com.ibm.icu.number.NumberFormatter;
 import com.ibm.icu.util.Output;
 import com.ibm.icu.util.ULocale;
 
@@ -172,35 +177,26 @@ public class PluralRules implements Serializable {
     static final UnicodeSet ALLOWED_ID = new UnicodeSet("[a-z]").freeze();
 
     // TODO Remove RulesList by moving its API and fields into PluralRules.
-    /**
-     * @internal
-     * @deprecated This API is ICU internal only.
-     */
-    @Deprecated
-    public static final String CATEGORY_SEPARATOR = ";  ";
-    /**
-     * @internal
-     * @deprecated This API is ICU internal only.
-     */
-    @Deprecated
-    public static final String KEYWORD_RULE_SEPARATOR = ": ";
+
+    private static final String CATEGORY_SEPARATOR = ";  ";
 
     private static final long serialVersionUID = 1;
 
     private final RuleList rules;
     private final transient Set<String> keywords;
+    private final transient StandardPluralRanges standardPluralRanges;
 
     /**
      * Provides a factory for returning plural rules
      *
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
     public static abstract class Factory {
         /**
          * Sole constructor
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -221,7 +217,7 @@ public class PluralRules implements Serializable {
          * @return The predefined <code>PluralRules</code> object for this locale. If there's no predefined rules for
          *         this locale, the rules for the closest parent in the locale hierarchy that has one will be returned.
          *         The final fallback always returns the default rules.
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -231,7 +227,7 @@ public class PluralRules implements Serializable {
          * Utility for getting CARDINAL rules.
          * @param locale the locale
          * @return plural rules.
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -242,7 +238,7 @@ public class PluralRules implements Serializable {
         /**
          * Returns the locales for which there is plurals data.
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -261,7 +257,7 @@ public class PluralRules implements Serializable {
          *            if not null and of length &gt; 0, this will hold 'true' at index 0 if locale is directly defined
          *            (without fallback) as having plural rules
          * @return the functionally-equivalent locale
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -269,7 +265,7 @@ public class PluralRules implements Serializable {
 
         /**
          * Returns the default factory.
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -279,7 +275,7 @@ public class PluralRules implements Serializable {
 
         /**
          * Returns whether or not there are overrides.
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -385,9 +381,7 @@ public class PluralRules implements Serializable {
      */
     public static PluralRules parseDescription(String description)
             throws ParseException {
-
-        description = description.trim();
-        return description.length() == 0 ? DEFAULT : new PluralRules(parseRuleChain(description));
+        return newInternal(description, null);
     }
 
     /**
@@ -405,15 +399,29 @@ public class PluralRules implements Serializable {
         }
     }
 
+    /**
+     * @internal
+     * @deprecated This API is ICU internal only.
+     */
+    @Deprecated
+    public static PluralRules newInternal(String description, StandardPluralRanges ranges)
+            throws ParseException {
+        description = description.trim();
+        return description.length() == 0
+            ? DEFAULT
+            : new PluralRules(parseRuleChain(description), ranges);
+    }
+
     /**
      * The default rules that accept any number and return
      * {@link #KEYWORD_OTHER}.
      * @stable ICU 3.8
      */
-    public static final PluralRules DEFAULT = new PluralRules(new RuleList().addRule(DEFAULT_RULE));
+    public static final PluralRules DEFAULT = new PluralRules(
+        new RuleList().addRule(DEFAULT_RULE), StandardPluralRanges.DEFAULT);
 
     /**
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -421,7 +429,7 @@ public class PluralRules implements Serializable {
         /**
          * The double value of the entire number.
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -430,7 +438,7 @@ public class PluralRules implements Serializable {
         /**
          * The integer value, with the fraction digits truncated off.
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -439,7 +447,7 @@ public class PluralRules implements Serializable {
         /**
          * All visible fraction digits as an integer, including trailing zeros.
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -448,7 +456,7 @@ public class PluralRules implements Serializable {
         /**
          * Visible fraction digits as an integer, not including trailing zeros.
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -457,7 +465,7 @@ public class PluralRules implements Serializable {
         /**
          * Number of visible fraction digits.
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -466,12 +474,22 @@ public class PluralRules implements Serializable {
         /**
          * Number of visible fraction digits, not including trailing zeros.
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         w,
 
+        /**
+         * Suppressed exponent for compact notation (exponent needed in
+         * scientific notation with compact notation to approximate i).
+         *
+         * @internal
+         * @deprecated This API is ICU internal only.
+         */
+        @Deprecated
+        e,
+
         /**
          * THIS OPERAND IS DEPRECATED AND HAS BEEN REMOVED FROM THE SPEC.
          *
@@ -481,7 +499,7 @@ public class PluralRules implements Serializable {
          * <p>For example, "j is 3" is equivalent to "i is 3 and v is 0": it matches
          * "3" but not "3.1" or "3.0".
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -491,7 +509,7 @@ public class PluralRules implements Serializable {
     /**
      * An interface to FixedDecimal, allowing for other implementations.
      *
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -500,21 +518,21 @@ public class PluralRules implements Serializable {
          * Returns the value corresponding to the specified operand (n, i, f, t, v, or w).
          * If the operand is 'n', returns a double; otherwise, returns an integer.
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         public double getPluralOperand(Operand operand);
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         public boolean isNaN();
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -522,7 +540,7 @@ public class PluralRules implements Serializable {
     }
 
     /**
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -545,10 +563,12 @@ public class PluralRules implements Serializable {
 
         final boolean isNegative;
 
+        final int exponent;
+
         private final int baseFactor;
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -557,7 +577,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -566,7 +586,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -575,7 +595,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -584,7 +604,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -593,7 +613,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -602,7 +622,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -611,7 +631,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -620,7 +640,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -631,15 +651,16 @@ public class PluralRules implements Serializable {
         static final long MAX = (long)1E18;
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          * @param n is the original number
          * @param v number of digits to the right of the decimal place. e.g 1.00 = 2 25. = 0
          * @param f Corresponds to f in the plural rules grammar.
          *   The digits to the right of the decimal place as an integer. e.g 1.10 = 10
+         * @param e Suppressed exponent for scientific and compact notation
          */
         @Deprecated
-        public FixedDecimal(double n, int v, long f) {
+        public FixedDecimal(double n, int v, long f, int e) {
             isNegative = n < 0;
             source = isNegative ? -n : n;
             visibleDecimalDigitCount = v;
@@ -647,6 +668,7 @@ public class PluralRules implements Serializable {
             integerValue = n > MAX
                     ? MAX
                             : (long)n;
+            exponent = e;
             hasIntegerValue = source == integerValue;
             // check values. TODO make into unit test.
             //
@@ -678,7 +700,25 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
+         * @deprecated This API is ICU internal only.
+         */
+        @Deprecated
+        public FixedDecimal(double n, int v, long f) {
+            this(n, v, f, 0);
+        }
+
+        /**
+         * @internal CLDR
+         * @deprecated This API is ICU internal only.
+         */
+        @Deprecated
+        public static FixedDecimal createWithExponent(double n, int v, int e) {
+            return new FixedDecimal(n,v,getFractionalDigits(n, v), e);
+        }
+
+        /**
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -700,7 +740,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -709,7 +749,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -722,7 +762,7 @@ public class PluralRules implements Serializable {
          * Return a guess as to the number of decimals that would be displayed. This is only a guess; callers should
          * always supply the decimals explicitly if possible. Currently, it is up to 6 decimals (without trailing zeros).
          * Returns 0 for infinities and nans.
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          *
          */
@@ -770,13 +810,57 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
+         * @deprecated This API is ICU internal only
+         */
+        @Deprecated
+        private FixedDecimal (FixedDecimal other) {
+            // Ugly, but necessary, because constructors must only call other
+            // constructors in the first line of the body, and
+            // FixedDecimal(String) was refactored to support exponents.
+            this.source = other.source;
+            this.visibleDecimalDigitCount = other.visibleDecimalDigitCount;
+            this.visibleDecimalDigitCountWithoutTrailingZeros =
+                    other.visibleDecimalDigitCountWithoutTrailingZeros;
+            this.decimalDigits = other.decimalDigits;
+            this.decimalDigitsWithoutTrailingZeros =
+                    other.decimalDigitsWithoutTrailingZeros;
+            this.integerValue = other.integerValue;
+            this.hasIntegerValue = other.hasIntegerValue;
+            this.isNegative = other.isNegative;
+            this.exponent = other.exponent;
+            this.baseFactor = other.baseFactor;
+        }
+
+        /**
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         public FixedDecimal (String n) {
             // Ugly, but for samples we don't care.
-            this(Double.parseDouble(n), getVisibleFractionCount(n));
+            this(parseDecimalSampleRangeNumString(n));
+        }
+
+        /**
+         * @internal CLDR
+         * @deprecated This API is ICU internal only
+         */
+        @Deprecated
+        private static FixedDecimal parseDecimalSampleRangeNumString(String num) {
+            if (num.contains("e")) {
+                int ePos = num.lastIndexOf('e');
+                int expNumPos = ePos + 1;
+                String exponentStr = num.substring(expNumPos);
+                int exponent = Integer.parseInt(exponentStr);
+                String fractionStr = num.substring(0, ePos);
+                return FixedDecimal.createWithExponent(
+                        Double.parseDouble(fractionStr),
+                        getVisibleFractionCount(fractionStr),
+                        exponent);
+            } else {
+                return new FixedDecimal(Double.parseDouble(num), getVisibleFractionCount(num));
+            }
         }
 
         private static int getVisibleFractionCount(String value) {
@@ -792,7 +876,7 @@ public class PluralRules implements Serializable {
         /**
          * {@inheritDoc}
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Override
@@ -805,12 +889,13 @@ public class PluralRules implements Serializable {
             case t: return decimalDigitsWithoutTrailingZeros;
             case v: return visibleDecimalDigitCount;
             case w: return visibleDecimalDigitCountWithoutTrailingZeros;
+            case e: return exponent;
             default: return source;
             }
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -820,12 +905,15 @@ public class PluralRules implements Serializable {
 
         /**
          * We're not going to care about NaN.
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Override
         @Deprecated
         public int compareTo(FixedDecimal other) {
+            if (exponent != other.exponent) {
+                return doubleValue() < other.doubleValue() ? -1 : 1;
+            }
             if (integerValue != other.integerValue) {
                 return integerValue < other.integerValue ? -1 : 1;
             }
@@ -843,7 +931,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -859,11 +947,12 @@ public class PluralRules implements Serializable {
                 return false;
             }
             FixedDecimal other = (FixedDecimal)arg0;
-            return source == other.source && visibleDecimalDigitCount == other.visibleDecimalDigitCount && decimalDigits == other.decimalDigits;
+            return source == other.source && visibleDecimalDigitCount == other.visibleDecimalDigitCount && decimalDigits == other.decimalDigits
+                    && exponent == other.exponent;
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -874,17 +963,22 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         @Override
         public String toString() {
-            return String.format("%." + visibleDecimalDigitCount + "f", source);
+            String baseString = String.format(Locale.ROOT, "%." + visibleDecimalDigitCount + "f", source);
+            if (exponent == 0) {
+                return baseString;
+            } else {
+                return baseString + "e" + exponent;
+            }
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -893,48 +987,52 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         @Override
         public int intValue() {
             // TODO Auto-generated method stub
-            return (int)integerValue;
+            return (int) longValue();
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         @Override
         public long longValue() {
-            return integerValue;
+            if (exponent == 0) {
+                return integerValue;
+            } else {
+                return (long) (Math.pow(10, exponent) * integerValue);
+            }
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         @Override
         public float floatValue() {
-            return (float) source;
+            return (float) (source * Math.pow(10, exponent));
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         @Override
         public double doubleValue() {
-            return isNegative ? -source : source;
+            return (isNegative ? -source : source) * Math.pow(10, exponent);
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -956,7 +1054,7 @@ public class PluralRules implements Serializable {
         /**
          * {@inheritDoc}
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -968,7 +1066,7 @@ public class PluralRules implements Serializable {
         /**
          * {@inheritDoc}
          *
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -980,19 +1078,19 @@ public class PluralRules implements Serializable {
 
     /**
      * Selection parameter for either integer-only or decimal-only.
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
     public enum SampleType {
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         INTEGER,
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -1001,25 +1099,25 @@ public class PluralRules implements Serializable {
 
     /**
      * A range of NumberInfo that includes all values with the same visibleFractionDigitCount.
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
     public static class FixedDecimalRange {
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         public final FixedDecimal start;
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         public final FixedDecimal end;
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -1031,7 +1129,7 @@ public class PluralRules implements Serializable {
             this.end = end;
         }
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -1043,25 +1141,25 @@ public class PluralRules implements Serializable {
 
     /**
      * A list of NumberInfo that includes all values with the same visibleFractionDigitCount.
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
     public static class FixedDecimalSamples {
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         public final SampleType sampleType;
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
         public final Set<FixedDecimalRange> samples;
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -1084,7 +1182,7 @@ public class PluralRules implements Serializable {
             SampleType sampleType2;
             boolean bounded2 = true;
             boolean haveBound = false;
-            Set<FixedDecimalRange> samples2 = new LinkedHashSet<FixedDecimalRange>();
+            Set<FixedDecimalRange> samples2 = new LinkedHashSet<>();
 
             if (source.startsWith("integer")) {
                 sampleType2 = SampleType.INTEGER;
@@ -1131,7 +1229,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -1149,7 +1247,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -1172,7 +1270,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -1181,7 +1279,7 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
+         * @internal CLDR
          * @deprecated This API is ICU internal only.
          */
         @Deprecated
@@ -1215,7 +1313,7 @@ public class PluralRules implements Serializable {
         static final UnicodeSet BREAK_AND_KEEP = new UnicodeSet('!', '!', '%', '%', ',', ',', '.', '.', '=', '=').freeze();
         static String[] split(String source) {
             int last = -1;
-            List<String> result = new ArrayList<String>();
+            List<String> result = new ArrayList<>();
             for (int i = 0; i < source.length(); ++i) {
                 char ch = source.charAt(i);
                 if (BREAK_AND_IGNORE.contains(ch)) {
@@ -1334,7 +1432,7 @@ public class PluralRules implements Serializable {
                         t = nextToken(tokens, x++, condition);
                     }
 
-                    List<Long> valueList = new ArrayList<Long>();
+                    List<Long> valueList = new ArrayList<>();
 
                     // the token t is always one item ahead
                     while (true) {
@@ -1756,10 +1854,9 @@ public class PluralRules implements Serializable {
         }
 
         /**
-         * @internal
-         * @deprecated This API is ICU internal only.
+         * {@inheritDoc}
+         * @stable ICU 3.8
          */
-        @Deprecated
         @Override
         public int hashCode() {
             return keyword.hashCode() ^ constraint.hashCode();
@@ -1773,7 +1870,7 @@ public class PluralRules implements Serializable {
     private static class RuleList implements Serializable {
         private boolean hasExplicitBoundingInfo = false;
         private static final long serialVersionUID = 1;
-        private final List<Rule> rules = new ArrayList<Rule>();
+        private final List<Rule> rules = new ArrayList<>();
 
         public RuleList addRule(Rule nextRule) {
             String keyword = nextRule.getKeyword();
@@ -1821,7 +1918,7 @@ public class PluralRules implements Serializable {
         }
 
         public Set<String> getKeywords() {
-            Set<String> result = new LinkedHashSet<String>();
+            Set<String> result = new LinkedHashSet<>();
             for (Rule rule : rules) {
                 result.add(rule.getKeyword());
             }
@@ -2014,23 +2111,24 @@ public class PluralRules implements Serializable {
     /*
      * Creates a new <code>PluralRules</code> object.  Immutable.
      */
-    private PluralRules(RuleList rules) {
+    private PluralRules(RuleList rules, StandardPluralRanges standardPluralRanges) {
         this.rules = rules;
         this.keywords = Collections.unmodifiableSet(rules.getKeywords());
+        this.standardPluralRanges = standardPluralRanges;
     }
 
     /**
-     * @internal
-     * @deprecated This API is ICU internal only.
+     * {@inheritDoc}
+     * @stable ICU 3.8
      */
-    @Deprecated
     @Override
     public int hashCode() {
         return rules.hashCode();
     }
+
     /**
-     * Given a number, returns the keyword of the first rule that applies to
-     * the number.
+     * Given a floating-point number, returns the keyword of the first rule
+     * that applies to the number.
      *
      * @param number The number for which the rule has to be determined.
      * @return The keyword of the selected rule.
@@ -2040,13 +2138,57 @@ public class PluralRules implements Serializable {
         return rules.select(new FixedDecimal(number));
     }
 
+    /**
+     * Given a formatted number, returns the keyword of the first rule that
+     * applies to the number.
+     *
+     * A FormattedNumber allows you to specify an exponent or trailing zeros,
+     * which can affect the plural category. To get a FormattedNumber, see
+     * {@link NumberFormatter}.
+     *
+     * @param number The number for which the rule has to be determined.
+     * @return The keyword of the selected rule.
+     * @stable ICU 64
+     */
+    public String select(FormattedNumber number) {
+        return rules.select(number.getFixedDecimal());
+    }
+
+    /**
+     * Given a formatted number range, returns the overall plural form of the
+     * range. For example, "3-5" returns "other" in English.
+     *
+     * To get a FormattedNumberRange, see {@link com.ibm.icu.number.NumberRangeFormatter}.
+     *
+     * This method only works if PluralRules was created with a locale. If it was created
+     * from PluralRules.createRules(), or if it was deserialized, this method throws
+     * UnsupportedOperationException.
+     *
+     * @param range  The number range onto which the rules will be applied.
+     * @return       The keyword of the selected rule.
+     * @throws UnsupportedOperationException If called on an instance without plural ranges data.
+     * @draft ICU 68
+     * @provisional This API might change or be removed in a future release.
+     */
+    public String select(FormattedNumberRange range) {
+        if (standardPluralRanges == null) {
+            throw new UnsupportedOperationException("Plural ranges are unavailable on this instance");
+        }
+        StandardPlural form1 = StandardPlural.fromString(
+            select(range.getFirstFixedDecimal()));
+        StandardPlural form2 = StandardPlural.fromString(
+            select(range.getSecondFixedDecimal()));
+        StandardPlural result = standardPluralRanges.resolve(form1, form2);
+        return result.getKeyword();
+    }
+
     /**
      * Given a number, returns the keyword of the first rule that applies to
      * the number.
      *
      * @param number The number for which the rule has to be determined.
      * @return The keyword of the selected rule.
-     * @internal
+     * @internal Visible For Testing
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -2060,7 +2202,7 @@ public class PluralRules implements Serializable {
      *
      * @param number The number information for which the rule has to be determined.
      * @return The keyword of the selected rule.
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -2073,7 +2215,7 @@ public class PluralRules implements Serializable {
      *
      * @param sample The number information for which the rule has to be determined.
      * @param keyword The keyword to filter on
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -2130,7 +2272,7 @@ public class PluralRules implements Serializable {
      * @return the values that trigger this keyword, or null.  The returned collection
      * is immutable. It will be empty if the keyword is not defined.
      *
-     * @internal
+     * @internal Visible For Testing
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -2167,7 +2309,7 @@ public class PluralRules implements Serializable {
      * @param keyword the keyword to test
      * @param sampleType the type of samples requested, INTEGER or DECIMAL
      * @return a list of values matching the keyword.
-     * @internal
+     * @internal CLDR
      * @deprecated ICU internal only
      */
     @Deprecated
@@ -2175,7 +2317,7 @@ public class PluralRules implements Serializable {
         if (!keywords.contains(keyword)) {
             return null;
         }
-        Set<Double> result = new TreeSet<Double>();
+        Set<Double> result = new TreeSet<>();
 
         if (rules.hasExplicitBoundingInfo) {
             FixedDecimalSamples samples = rules.getDecimalSamples(keyword, sampleType);
@@ -2207,12 +2349,7 @@ public class PluralRules implements Serializable {
         return result.size() == 0 ? null : Collections.unmodifiableSet(result);
     }
 
-    /**
-     * @internal
-     * @deprecated This API is ICU internal only.
-     */
-    @Deprecated
-    public boolean addSample(String keyword, Number sample, int maxCount, Set<Double> result) {
+    private boolean addSample(String keyword, Number sample, int maxCount, Set<Double> result) {
         String selectedKeyword = sample instanceof FixedDecimal ? select((FixedDecimal)sample) : select(sample.doubleValue());
         if (selectedKeyword.equals(keyword)) {
             result.add(sample.doubleValue());
@@ -2233,7 +2370,7 @@ public class PluralRules implements Serializable {
      * @param keyword the keyword to test
      * @param sampleType the type of samples requested, INTEGER or DECIMAL
      * @return a list of values matching the keyword.
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -2380,7 +2517,7 @@ public class PluralRules implements Serializable {
      * @param uniqueValue
      *            If non null, set to the unique value.
      * @return the KeywordStatus
-     * @internal
+     * @internal Visible For Testing
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -2420,7 +2557,7 @@ public class PluralRules implements Serializable {
 
         // Compute if the quick test is insufficient.
 
-        HashSet<Double> subtractedSet = new HashSet<Double>(values);
+        HashSet<Double> subtractedSet = new HashSet<>(values);
         for (Double explicit : explicits) {
             subtractedSet.remove(explicit - offset);
         }
@@ -2436,7 +2573,7 @@ public class PluralRules implements Serializable {
     }
 
     /**
-     * @internal
+     * @internal CLDR
      * @deprecated This API is ICU internal only.
      */
     @Deprecated
@@ -2460,7 +2597,7 @@ public class PluralRules implements Serializable {
     }
 
     /**
-     * @internal
+     * @internal CLDR
      * @deprecated internal
      */
     @Deprecated
@@ -2468,17 +2605,12 @@ public class PluralRules implements Serializable {
         return toString().compareTo(other.toString());
     }
 
-    /**
-     * @internal
-     * @deprecated internal
-     */
-    @Deprecated
-    public Boolean isLimited(String keyword) {
+    Boolean isLimited(String keyword) {
         return rules.isLimited(keyword, SampleType.INTEGER);
     }
 
     /**
-     * @internal
+     * @internal Visible For Testing
      * @deprecated internal
      */
     @Deprecated
@@ -2487,7 +2619,7 @@ public class PluralRules implements Serializable {
     }
 
     /**
-     * @internal
+     * @internal CLDR
      * @deprecated internal
      */
     @Deprecated
