@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /**
  *******************************************************************************
  * Copyright (C) 2001-2016, International Business Machines Corporation and
@@ -64,7 +64,7 @@ public class Currency extends MeasureUnit {
 
     // Cache to save currency name trie
     private static ICUCache<ULocale, List<TextTrieMap<CurrencyStringInfo>>> CURRENCY_NAME_CACHE =
-        new SimpleCache<ULocale, List<TextTrieMap<CurrencyStringInfo>>>();
+        new SimpleCache<>();
 
     /**
      * Selector for getName() indicating a symbolic name for a
@@ -90,15 +90,38 @@ public class Currency extends MeasureUnit {
 
     /**
      * Selector for getName() indicating the narrow currency symbol.
-     * The narrow currency symbol is similar to the regular currency
-     * symbol, but it always takes the shortest form: for example,
-     * "$" instead of "US$" for USD in en-CA.
+     * <p>
+     * The narrow currency symbol is similar to the regular currency symbol,
+     * but it always takes the shortest form;
+     * for example, "$" instead of "US$" for USD in en-CA.
      *
-     * @draft ICU 61
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 61
      */
     public static final int NARROW_SYMBOL_NAME = 3;
 
+    /**
+     * Selector for getName() indicating the formal currency symbol.
+     * <p>
+     * The formal currency symbol is similar to the regular currency symbol,
+     * but it always takes the form used in formal settings such as banking;
+     * for example, "NT$" instead of "$" for TWD in zh-TW.
+     *
+     * @draft ICU 67
+     * @provisional This API might change or be removed in a future release.
+     */
+    public static final int FORMAL_SYMBOL_NAME = 4;
+
+    /**
+     * Selector for getName() indicating the variant currency symbol.
+     * <p>
+     * The variant symbol for a currency is an alternative symbol that is not
+     * necessarily as widely used as the regular symbol.
+     *
+     * @draft ICU 67
+     * @provisional This API might change or be removed in a future release.
+     */
+    public static final int VARIANT_SYMBOL_NAME = 5;
+
     /**
      * Currency Usage used for Decimal Format
      * @stable ICU 54
@@ -229,14 +252,13 @@ public class Currency extends MeasureUnit {
     public static Set<Currency> getAvailableCurrencies() {
         CurrencyMetaInfo info = CurrencyMetaInfo.getInstance();
         List<String> list = info.currencies(CurrencyFilter.all());
-        HashSet<Currency> resultSet = new HashSet<Currency>(list.size());
+        HashSet<Currency> resultSet = new HashSet<>(list.size());
         for (String code : list) {
             resultSet.add(getInstance(code));
         }
         return resultSet;
     }
 
-    private static final String EUR_STR = "EUR";
     private static final CacheBase<String, Currency, Void> regionCurrencyCache =
             new SoftCache<String, Currency, Void>() {
         @Override
@@ -249,40 +271,18 @@ public class Currency extends MeasureUnit {
      * Instantiate a currency from resource data.
      */
     /* package */ static Currency createCurrency(ULocale loc) {
-        String variant = loc.getVariant();
-        if ("EURO".equals(variant)) {
-            return getInstance(EUR_STR);
-        }
-
-        // Cache the currency by region, and whether variant=PREEURO.
+        // Cache the currency by region.
         // Minimizes the size of the cache compared with caching by ULocale.
         String key = ULocale.getRegionForSupplementalData(loc, false);
-        if ("PREEURO".equals(variant)) {
-            key = key + '-';
-        }
         return regionCurrencyCache.getInstance(key, null);
     }
 
     private static Currency loadCurrency(String key) {
-        String region;
-        boolean isPreEuro;
-        if (key.endsWith("-")) {
-            region = key.substring(0, key.length() - 1);
-            isPreEuro = true;
-        } else {
-            region = key;
-            isPreEuro = false;
-        }
+        String region = key;
         CurrencyMetaInfo info = CurrencyMetaInfo.getInstance();
         List<String> list = info.currencies(CurrencyFilter.onRegion(region));
         if (!list.isEmpty()) {
             String code = list.get(0);
-            if (isPreEuro && EUR_STR.equals(code)) {
-                if (list.size() < 2) {
-                    return null;
-                }
-                code = list.get(1);
-            }
             return getInstance(code);
         }
         return null;
@@ -328,7 +328,7 @@ public class Currency extends MeasureUnit {
      *
      * @param currency The Java currency object to convert.
      * @return An equivalent ICU currency object.
-     * @draft ICU 60
+     * @stable ICU 60
      */
     public static Currency fromJavaCurrency(java.util.Currency currency) {
         return getInstance(currency.getCurrencyCode());
@@ -338,7 +338,7 @@ public class Currency extends MeasureUnit {
      * Returns a java.util.Currency object based on the currency represented by this Currency.
      *
      * @return An equivalent Java currency object.
-     * @draft ICU 60
+     * @stable ICU 60
      */
     public java.util.Currency toJavaCurrency() {
         return java.util.Currency.getInstance(getCurrencyCode());
@@ -543,6 +543,17 @@ public class Currency extends MeasureUnit {
      * given locale.
      * This is a convenient method for
      * getName(ULocale, int, boolean[]);
+     *
+     * @param locale locale in which to display currency
+     * @param nameStyle selector for which kind of name to return.
+     *                  The nameStyle should be SYMBOL_NAME, NARROW_SYMBOL_NAME,
+     *                  or LONG_NAME. Otherwise, throw IllegalArgumentException.
+     * @param isChoiceFormat isChoiceFormat[0] is always set to false, or isChoiceFormat can be null;
+     *     display names are static strings;
+     *     since ICU 4.4, ChoiceFormat patterns are no longer supported
+     * @return display string for this currency.  If the resource data
+     * contains no entry for this currency, then the ISO 4217 code is
+     * returned.
      * @stable ICU 3.2
      */
     public String getName(Locale locale,
@@ -555,19 +566,17 @@ public class Currency extends MeasureUnit {
      * Returns the display name for the given currency in the
      * given locale.  For example, the display name for the USD
      * currency object in the en_US locale is "$".
+     *
      * @param locale locale in which to display currency
      * @param nameStyle selector for which kind of name to return.
      *                  The nameStyle should be SYMBOL_NAME, NARROW_SYMBOL_NAME,
      *                  or LONG_NAME. Otherwise, throw IllegalArgumentException.
-     * @param isChoiceFormat fill-in; isChoiceFormat[0] is set to true
-     * if the returned value is a ChoiceFormat pattern; otherwise it
-     * is set to false
+     * @param isChoiceFormat isChoiceFormat[0] is always set to false, or isChoiceFormat can be null;
+     *     display names are static strings;
+     *     since ICU 4.4, ChoiceFormat patterns are no longer supported
      * @return display string for this currency.  If the resource data
      * contains no entry for this currency, then the ISO 4217 code is
-     * returned.  If isChoiceFormat[0] is true, then the result is a
-     * ChoiceFormat pattern.  Otherwise it is a static string. <b>Note:</b>
-     * as of ICU 4.4, choice formats are not used, and the value returned
-     * in isChoiceFormat is always false.
+     * returned.
      * <p>
      * @throws  IllegalArgumentException  if the nameStyle is not SYMBOL_NAME
      *                                    or LONG_NAME.
@@ -587,6 +596,10 @@ public class Currency extends MeasureUnit {
             return names.getSymbol(subType);
         case NARROW_SYMBOL_NAME:
             return names.getNarrowSymbol(subType);
+        case FORMAL_SYMBOL_NAME:
+            return names.getFormalSymbol(subType);
+        case VARIANT_SYMBOL_NAME:
+            return names.getVariantSymbol(subType);
         case LONG_NAME:
             return names.getName(subType);
         default:
@@ -597,6 +610,16 @@ public class Currency extends MeasureUnit {
     /**
      * Returns the display name for the given currency in the given locale.
      * This is a convenience overload of getName(ULocale, int, String, boolean[]);
+     *
+     * @param locale locale in which to display currency
+     * @param nameStyle selector for which kind of name to return
+     * @param pluralCount plural count string for this locale
+     * @param isChoiceFormat isChoiceFormat[0] is always set to false, or isChoiceFormat can be null;
+     *     display names are static strings;
+     *     since ICU 4.4, ChoiceFormat patterns are no longer supported
+     * @return display string for this currency.  If the resource data
+     * contains no entry for this currency, then the ISO 4217 code is
+     * returned.
      * @stable ICU 4.2
      */
     public String getName(Locale locale, int nameStyle, String pluralCount,
@@ -612,18 +635,16 @@ public class Currency extends MeasureUnit {
      * amount is plural is "US dollars", such as in "3.00 US dollars";
      * while the PLURAL_LONG_NAME for the USD currency object when the currency
      * amount is singular is "US dollar", such as in "1.00 US dollar".
+     *
      * @param locale locale in which to display currency
      * @param nameStyle selector for which kind of name to return
      * @param pluralCount plural count string for this locale
-     * @param isChoiceFormat fill-in; isChoiceFormat[0] is set to true
-     * if the returned value is a ChoiceFormat pattern; otherwise it
-     * is set to false
+     * @param isChoiceFormat isChoiceFormat[0] is always set to false, or isChoiceFormat can be null;
+     *     display names are static strings;
+     *     since ICU 4.4, ChoiceFormat patterns are no longer supported
      * @return display string for this currency.  If the resource data
      * contains no entry for this currency, then the ISO 4217 code is
-     * returned.  If isChoiceFormat[0] is true, then the result is a
-     * ChoiceFormat pattern.  Otherwise it is a static string. <b>Note:</b>
-     * as of ICU 4.4, choice formats are not used, and the value returned
-     * in isChoiceFormat is always false.
+     * returned.
      * @throws  IllegalArgumentException  if the nameStyle is not SYMBOL_NAME,
      *                                    LONG_NAME, or PLURAL_LONG_NAME.
      * @stable ICU 4.2
@@ -657,7 +678,6 @@ public class Currency extends MeasureUnit {
      * @see #getName(Locale, int, boolean[])
      * @stable ICU 49
      */
-    @SuppressWarnings("javadoc")    // java.util.Currency#getDisplayName() is introduced in Java 7
     public String getDisplayName() {
         return getName(Locale.getDefault(), LONG_NAME, null);
     }
@@ -677,7 +697,6 @@ public class Currency extends MeasureUnit {
      * @see #getName(Locale, int, boolean[])
      * @stable ICU 49
      */
-    @SuppressWarnings("javadoc")    // java.util.Currency#getDisplayName() is introduced in Java 7
     public String getDisplayName(Locale locale) {
         return getName(locale, LONG_NAME, null);
     }
@@ -748,10 +767,10 @@ public class Currency extends MeasureUnit {
         List<TextTrieMap<CurrencyStringInfo>> currencyTrieVec = CURRENCY_NAME_CACHE.get(locale);
         if (currencyTrieVec == null) {
             TextTrieMap<CurrencyStringInfo> currencyNameTrie =
-                new TextTrieMap<CurrencyStringInfo>(true);
+                new TextTrieMap<>(true);
             TextTrieMap<CurrencyStringInfo> currencySymbolTrie =
-                new TextTrieMap<CurrencyStringInfo>(false);
-            currencyTrieVec = new ArrayList<TextTrieMap<CurrencyStringInfo>>();
+                new TextTrieMap<>(false);
+            currencyTrieVec = new ArrayList<>();
             currencyTrieVec.add(currencySymbolTrie);
             currencyTrieVec.add(currencyNameTrie);
             setupCurrencyTrieVec(locale, currencyTrieVec);
@@ -864,6 +883,13 @@ public class Currency extends MeasureUnit {
      * Returns the number of the number of fraction digits that should
      * be displayed for this currency.
      * This is equivalent to getDefaultFractionDigits(CurrencyUsage.STANDARD);
+     *
+     * Important: The number of fraction digits for a given currency is NOT
+     * guaranteed to be constant across versions of ICU or CLDR. For example,
+     * do NOT use this value as a mechanism for deciding the magnitude used
+     * to store currency values in a database. You should use this value for
+     * display purposes only.
+     *
      * @return a non-negative number of fraction digits to be
      * displayed
      * @stable ICU 2.2
@@ -875,6 +901,13 @@ public class Currency extends MeasureUnit {
     /**
      * Returns the number of the number of fraction digits that should
      * be displayed for this currency with Usage.
+     *
+     * Important: The number of fraction digits for a given currency is NOT
+     * guaranteed to be constant across versions of ICU or CLDR. For example,
+     * do NOT use this value as a mechanism for deciding the magnitude used
+     * to store currency values in a database. You should use this value for
+     * display purposes only.
+     *
      * @param Usage the usage of currency(Standard or Cash)
      * @return a non-negative number of fraction digits to be
      * displayed
@@ -971,7 +1004,7 @@ public class Currency extends MeasureUnit {
             //CurrencyFilter filter = CurrencyFilter.onDateRange(null, new Date(253373299200000L));
             CurrencyFilter filter = CurrencyFilter.all();
             all = Collections.unmodifiableList(getTenderCurrencies(filter));
-            ALL_TENDER_CODES = new SoftReference<List<String>>(all);
+            ALL_TENDER_CODES = new SoftReference<>(all);
         }
         return all;
     }
@@ -981,8 +1014,8 @@ public class Currency extends MeasureUnit {
         if (all == null) {
             CurrencyMetaInfo info = CurrencyMetaInfo.getInstance();
             all = Collections.unmodifiableSet(
-                    new HashSet<String>(info.currencies(CurrencyFilter.all())));
-            ALL_CODES_AS_SET = new SoftReference<Set<String>>(all);
+                    new HashSet<>(info.currencies(CurrencyFilter.all())));
+            ALL_CODES_AS_SET = new SoftReference<>(all);
         }
         return all;
     }
