@@ -1,5 +1,5 @@
 // © 2017 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 package com.ibm.icu.dev.impl.number;
 
 import java.math.BigDecimal;
@@ -9,6 +9,7 @@ import java.text.FieldPosition;
 
 import com.ibm.icu.impl.StandardPlural;
 import com.ibm.icu.impl.number.DecimalQuantity;
+import com.ibm.icu.impl.number.Modifier.Signum;
 import com.ibm.icu.text.PluralRules;
 import com.ibm.icu.text.PluralRules.Operand;
 import com.ibm.icu.text.UFieldPosition;
@@ -94,6 +95,8 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
     1000000000000000000L
   };
 
+  private int origPrimaryScale;
+
   @Override
   public int maxRepresentableDigits() {
     return Integer.MAX_VALUE;
@@ -109,6 +112,7 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
     primaryScale = 0;
     primaryPrecision = computePrecision(primary);
     fallback = null;
+    origPrimaryScale = primaryScale;
   }
 
   /**
@@ -188,6 +192,8 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
       primary = -1;
       fallback = new BigDecimal(temp);
     }
+
+    origPrimaryScale = primaryScale;
   }
 
   static final double LOG_2_OF_TEN = 3.32192809489;
@@ -278,6 +284,7 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
     primaryPrecision = _other.primaryPrecision;
     fallback = _other.fallback;
     flags = _other.flags;
+    origPrimaryScale = _other.origPrimaryScale;
   }
 
   @Override
@@ -319,37 +326,39 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
   }
 
   @Override
-  public void setIntegerLength(int minInt, int maxInt) {
+  public void setMinInteger(int minInt) {
     // Graceful failures for bogus input
     minInt = Math.max(0, minInt);
-    maxInt = Math.max(0, maxInt);
-
-    // The minima must be less than or equal to the maxima
-    if (maxInt < minInt) {
-      minInt = maxInt;
-    }
 
     // Save values into internal state
     // Negation is safe for minFrac/maxFrac because -Integer.MAX_VALUE > Integer.MIN_VALUE
-    lOptPos = maxInt;
     lReqPos = minInt;
   }
 
   @Override
-  public void setFractionLength(int minFrac, int maxFrac) {
+  public void setMinFraction(int minFrac) {
     // Graceful failures for bogus input
     minFrac = Math.max(0, minFrac);
-    maxFrac = Math.max(0, maxFrac);
-
-    // The minima must be less than or equal to the maxima
-    if (maxFrac < minFrac) {
-      minFrac = maxFrac;
-    }
 
     // Save values into internal state
     // Negation is safe for minFrac/maxFrac because -Integer.MAX_VALUE > Integer.MIN_VALUE
     rReqPos = -minFrac;
-    rOptPos = -maxFrac;
+  }
+
+  @Override
+  public void applyMaxInteger(int maxInt) {
+    BigDecimal d;
+    if (primary != -1) {
+      d = BigDecimal.valueOf(primary).scaleByPowerOfTen(primaryScale);
+    } else {
+      d = fallback;
+    }
+    d = d.scaleByPowerOfTen(-maxInt).remainder(BigDecimal.ONE).scaleByPowerOfTen(maxInt);
+    if (primary != -1) {
+      primary = d.scaleByPowerOfTen(-primaryScale).longValueExact();
+    } else {
+      fallback = d;
+    }
   }
 
   @Override
@@ -363,6 +372,12 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
     primary = -1;
   }
 
+  @Override
+  public void roundToNickel(int roundingMagnitude, MathContext mathContext) {
+    BigDecimal nickel = BigDecimal.valueOf(5).scaleByPowerOfTen(roundingMagnitude);
+    roundToIncrement(nickel, mathContext);
+  }
+
   @Override
   public void roundToMagnitude(int roundingMagnitude, MathContext mathContext) {
     if (roundingMagnitude < -1000) {
@@ -455,7 +470,7 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
   }
 
   @Override
-  public boolean isZero() {
+  public boolean isZeroish() {
     if (primary == -1) {
       return fallback.compareTo(BigDecimal.ZERO) == 0;
     } else {
@@ -509,8 +524,18 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
   }
 
   @Override
-  public int signum() {
-      return isNegative() ? -1 : isZero() ? 0 : 1;
+  public Signum signum() {
+      boolean isZero = (isZeroish() && !isInfinite());
+      boolean isNeg = isNegative();
+      if (isZero && isNeg) {
+          return Signum.NEG_ZERO;
+      } else if (isZero) {
+          return Signum.POS_ZERO;
+      } else if (isNeg) {
+          return Signum.NEG;
+      } else {
+          return Signum.POS;
+      }
   }
 
   private void setNegative(boolean isNegative) {
@@ -873,9 +898,17 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
       if (isNegative()) {
           sb.append('-');
       }
-      for (int m = getUpperDisplayMagnitude(); m >= getLowerDisplayMagnitude(); m--) {
-        sb.append(getDigit(m));
-        if (m == 0) sb.append('.');
+      int upper = getUpperDisplayMagnitude();
+      int lower = getLowerDisplayMagnitude();
+      int p = upper;
+      for (; p >= 0; p--) {
+          sb.append((char) ('0' + getDigit(p)));
+      }
+      if (lower < 0) {
+          sb.append('.');
+      }
+      for(; p >= lower; p--) {
+          sb.append((char) ('0' + getDigit(p)));
       }
       return sb.toString();
   }
@@ -897,4 +930,14 @@ public class DecimalQuantity_SimpleStorage implements DecimalQuantity {
           .setFractionDigits((int) getPluralOperand(Operand.v), (long) getPluralOperand(Operand.f));
     }
   }
+
+  @Override
+  public int getExponent() {
+    return origPrimaryScale;
+  }
+
+  @Override
+  public void adjustExponent(int delta) {
+      origPrimaryScale = origPrimaryScale + delta;
+  }
 }
