@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  * Copyright (C) 1996-2016, Google, International Business Machines Corporation and
@@ -162,27 +162,27 @@ public class CompactDecimalFormatTest extends TestFmwk {
             {1234567890f, "12亿"},
             {12345678901f, "120亿"},
             {123456789012f, "1200亿"},
-            {1234567890123f, "1.2兆"},
-            {12345678901234f, "12兆"},
-            {123456789012345f, "120兆"},
+            {1234567890123f, "1.2万亿"},
+            {12345678901234f, "12万亿"},
+            {123456789012345f, "120万亿"},
     };
 
     Object[][] ChineseCurrencyTestData = {
-            {new CurrencyAmount(1f, Currency.getInstance("CNY")), "￥1"},
-            {new CurrencyAmount(12f, Currency.getInstance("CNY")), "￥12"},
-            {new CurrencyAmount(123f, Currency.getInstance("CNY")), "￥120"},
-            {new CurrencyAmount(1234f, Currency.getInstance("CNY")), "￥1200"},
-            {new CurrencyAmount(12345f, Currency.getInstance("CNY")), "￥1.2万"},
-            {new CurrencyAmount(123456f, Currency.getInstance("CNY")), "￥12万"},
-            {new CurrencyAmount(1234567f, Currency.getInstance("CNY")), "￥120万"},
-            {new CurrencyAmount(12345678f, Currency.getInstance("CNY")), "￥1200万"},
-            {new CurrencyAmount(123456789f, Currency.getInstance("CNY")), "￥1.2亿"},
-            {new CurrencyAmount(1234567890f, Currency.getInstance("CNY")), "￥12亿"},
-            {new CurrencyAmount(12345678901f, Currency.getInstance("CNY")), "￥120亿"},
-            {new CurrencyAmount(123456789012f, Currency.getInstance("CNY")), "￥1200亿"},
-            {new CurrencyAmount(1234567890123f, Currency.getInstance("CNY")), "￥1.2兆"},
-            {new CurrencyAmount(12345678901234f, Currency.getInstance("CNY")), "￥12兆"},
-            {new CurrencyAmount(123456789012345f, Currency.getInstance("CNY")), "￥120兆"},
+            {new CurrencyAmount(1f, Currency.getInstance("CNY")), "¥1"},
+            {new CurrencyAmount(12f, Currency.getInstance("CNY")), "¥12"},
+            {new CurrencyAmount(123f, Currency.getInstance("CNY")), "¥120"},
+            {new CurrencyAmount(1234f, Currency.getInstance("CNY")), "¥1200"},
+            {new CurrencyAmount(12345f, Currency.getInstance("CNY")), "¥1.2万"},
+            {new CurrencyAmount(123456f, Currency.getInstance("CNY")), "¥12万"},
+            {new CurrencyAmount(1234567f, Currency.getInstance("CNY")), "¥120万"},
+            {new CurrencyAmount(12345678f, Currency.getInstance("CNY")), "¥1200万"},
+            {new CurrencyAmount(123456789f, Currency.getInstance("CNY")), "¥1.2亿"},
+            {new CurrencyAmount(1234567890f, Currency.getInstance("CNY")), "¥12亿"},
+            {new CurrencyAmount(12345678901f, Currency.getInstance("CNY")), "¥120亿"},
+            {new CurrencyAmount(123456789012f, Currency.getInstance("CNY")), "¥1200亿"},
+            {new CurrencyAmount(1234567890123f, Currency.getInstance("CNY")), "¥1.2万亿"},
+            {new CurrencyAmount(12345678901234f, Currency.getInstance("CNY")), "¥12万亿"},
+            {new CurrencyAmount(123456789012345f, Currency.getInstance("CNY")), "¥120万亿"},
     };
     Object[][] GermanCurrencyTestData = {
             {new CurrencyAmount(1f, Currency.getInstance("EUR")), "1 €"},
@@ -226,15 +226,15 @@ public class CompactDecimalFormatTest extends TestFmwk {
             {1234f, "elfu\u00a01.2"},
             {12345f, "elfu\u00a012"},
             {123456f, "elfu\u00A0120"},
-            {1234567f, "M1.2"},
-            {12345678f, "M12"},
-            {123456789f, "M120"},
-            {1234567890f, "B1.2"},
-            {12345678901f, "B12"},
-            {123456789012f, "B120"},
-            {1234567890123f, "T1.2"},
-            {12345678901234f, "T12"},
-            {12345678901234567890f, "T12,000,000"},
+            {1234567f, "1.2M"},
+            {12345678f, "12M"},
+            {123456789f, "120M"},
+            {1234567890f, "1.2B"},
+            {12345678901f, "12B"},
+            {123456789012f, "120B"},
+            {1234567890123f, "1.2T"},
+            {12345678901234f, "12T"},
+            {12345678901234567890f, "12,000,000T"},
     };
 
     Object[][] CsTestDataShort = {
@@ -270,15 +270,15 @@ public class CompactDecimalFormatTest extends TestFmwk {
             {-1234f, "elfu\u00a0-1.2"},
             {-12345f, "elfu\u00a0-12"},
             {-123456f, "elfu\u00a0-120"},
-            {-1234567f, "M-1.2"},
-            {-12345678f, "M-12"},
-            {-123456789f, "M-120"},
-            {-1234567890f, "B-1.2"},
-            {-12345678901f, "B-12"},
-            {-123456789012f, "B-120"},
-            {-1234567890123f, "T-1.2"},
-            {-12345678901234f, "T-12"},
-            {-12345678901234567890f, "T-12,000,000"},
+            {-1234567f, "-1.2M"},
+            {-12345678f, "-12M"},
+            {-123456789f, "-120M"},
+            {-1234567890f, "-1.2B"},
+            {-12345678901f, "-12B"},
+            {-123456789012f, "-120B"},
+            {-1234567890123f, "-1.2T"},
+            {-12345678901234f, "-12T"},
+            {-12345678901234567890f, "-12,000,000T"},
     };
 
     Object[][] TestACoreCompactFormatList = {
@@ -377,12 +377,12 @@ public class CompactDecimalFormatTest extends TestFmwk {
         CompactDecimalFormat cdf =
                 getCDFInstance(ULocale.forLanguageTag("sw"), CompactStyle.SHORT);
         AttributedCharacterIterator iter = cdf.formatToCharacterIterator(1234567);
-        assertEquals("CharacterIterator", "M1.2", iterToString(iter));
+        assertEquals("CharacterIterator", "1.2M", iterToString(iter));
         iter = cdf.formatToCharacterIterator(1234567);
-        iter.setIndex(1);
+        iter.setIndex(0);
         assertEquals("Attributes", NumberFormat.Field.INTEGER, iter.getAttribute(NumberFormat.Field.INTEGER));
-        assertEquals("Attributes", 1, iter.getRunStart());
-        assertEquals("Attributes", 2, iter.getRunLimit());
+        assertEquals("Attributes", 0, iter.getRunStart());
+        assertEquals("Attributes", 1, iter.getRunLimit());
     }
 
     @Test
@@ -464,9 +464,9 @@ public class CompactDecimalFormatTest extends TestFmwk {
         FieldPosition fp = new FieldPosition(0);
         StringBuffer sb = new StringBuffer();
         cdf.format(1234567f, sb, fp);
-        assertEquals("fp string", "M1.2", sb.toString());
-        assertEquals("fp start", 1, fp.getBeginIndex());
-        assertEquals("fp end", 2, fp.getEndIndex());
+        assertEquals("fp string", "1.2M", sb.toString());
+        assertEquals("fp start", 0, fp.getBeginIndex());
+        assertEquals("fp end", 1, fp.getEndIndex());
     }
 
     @Test
@@ -698,9 +698,9 @@ public class CompactDecimalFormatTest extends TestFmwk {
         // Check currency formatting as well
         cdf = CompactDecimalFormat.getInstance(new ULocale("ar-EG"), CompactDecimalFormat.CompactStyle.SHORT);
         result = cdf.format(new CurrencyAmount(43000f, Currency.getInstance("USD")));
-        assertEquals("CDF should correctly format 43000 with currency in 'ar-EG'", "US$ ٤٣ ألف", result);
+        assertEquals("CDF should correctly format 43000 with currency in 'ar-EG'", "٤٣ ألف US$", result);
         result = cdf.format(new CurrencyAmount(-43000f, Currency.getInstance("USD")));
-        assertEquals("CDF should correctly format -43000 with currency in 'ar-EG'", "؜-US$ ٤٣ ألف", result);
+        assertEquals("CDF should correctly format -43000 with currency in 'ar-EG'", "؜-٤٣ ألف US$", result);
 
         // Extra locale with different positive/negative formats
         cdf = CompactDecimalFormat.getInstance(new ULocale("fi"), CompactDecimalFormat.CompactStyle.SHORT);
@@ -713,54 +713,42 @@ public class CompactDecimalFormatTest extends TestFmwk {
 
     @Test
     public void TestBug12689() {
-        if (logKnownIssue("12689", "CDF fails for numbers less than 1 thousand in most locales")) {
-            return;
-        }
-
         CompactDecimalFormat cdf;
         String result;
 
         cdf = CompactDecimalFormat.getInstance(ULocale.forLanguageTag("en"), CompactStyle.SHORT);
         result = cdf.format(new CurrencyAmount(123, Currency.getInstance("USD")));
-        assertEquals("CDF should correctly format 123 with currency in 'en'", "$120", result);
+        assertEquals("CDF should correctly format 123 with currency in 'en'", "$123", result);
 
         cdf = CompactDecimalFormat.getInstance(ULocale.forLanguageTag("it"), CompactStyle.SHORT);
         result = cdf.format(new CurrencyAmount(123, Currency.getInstance("EUR")));
-        assertEquals("CDF should correctly format 123 with currency in 'it'", "120 €", result);
+        assertEquals("CDF should correctly format 123 with currency in 'it'", "123\u00A0€", result);
     }
 
     @Test
     public void TestBug12688() {
-        if (logKnownIssue("12688", "CDF fails for numbers less than 1 million in 'it'")) {
-            return;
-        }
-
         CompactDecimalFormat cdf;
         String result;
 
         cdf = CompactDecimalFormat.getInstance(ULocale.forLanguageTag("it"), CompactStyle.SHORT);
         result = cdf.format(new CurrencyAmount(123000, Currency.getInstance("EUR")));
-        assertEquals("CDF should correctly format 123000 with currency in 'it'", "120000 €", result);
+        assertEquals("CDF should correctly format 123000 with currency in 'it'", "123.000\u00A0€", result);
     }
 
     @Test
     public void TestBug11319() {
-        if (logKnownIssue("11319", "CDF does not fall back from zh-Hant-HK to zh-Hant")) {
-            return;
-        }
-
-        CompactDecimalFormat cdf = CompactDecimalFormat.getInstance(new ULocale("zh-Hant-HK"), CompactStyle.SHORT);
+        CompactDecimalFormat cdf = CompactDecimalFormat.getInstance(new ULocale("yue-HK"), CompactStyle.SHORT);
         String result = cdf.format(958000000L);
-        assertEquals("CDF should correctly format 958 million in zh-Hant-HK", "9.6億", result);
+        assertEquals("CDF should correctly format 958 million in yue-HK", "9.6億", result);
     }
 
     @Test
     public void TestBug12975() {
         ULocale locale = new ULocale("it");
         CompactDecimalFormat cdf = CompactDecimalFormat.getInstance(locale, CompactStyle.SHORT);
-        String resultCdf = cdf.format(120000);
+        String resultCdf = cdf.format(12000);
         DecimalFormat df = (DecimalFormat) DecimalFormat.getInstance(locale);
-        String resultDefault = df.format(120000);
+        String resultDefault = df.format(12000);
         assertEquals("CompactDecimalFormat should use default pattern when compact pattern is unavailable",
                      resultDefault, resultCdf);
     }
