@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  * Copyright (C) 2001-2015, International Business Machines Corporation and    *
@@ -437,7 +437,9 @@ public class DateFormatRegressionTest extends TestFmwk {
     public void Test4089106() {
         TimeZone def = TimeZone.getDefault();
         try {
-            TimeZone z = new SimpleTimeZone((int) (1.25 * 3600000), "FAKEZONE");
+            // It's necessary to use a real existing time zone here, some systems (Android) will not
+            // accept any arbitrary TimeZone object to be used as the default.
+            TimeZone z = new SimpleTimeZone(-12 * 60 * 60 * 1000, "GMT-12:00");
             TimeZone.setDefault(z);
             SimpleDateFormat f = new SimpleDateFormat();
             if (!f.getTimeZone().equals(z))
@@ -1236,6 +1238,38 @@ public class DateFormatRegressionTest extends TestFmwk {
         assertEquals("Bad date format", "Mo1 20, 2013", dangiDateStr);
     }
 
+    @Test
+    public void Test12902_yWithGregoCalInThaiLoc() {
+        final Date testDate = new Date(43200000); // 1970-Jan-01 12:00 GMT
+        final String skeleton = "y";
+        // Note that in locale "th", the availableFormats for skeleton "y" differ by calendar:
+        // for buddhist (default calendar): y{"G y"}
+        // for gregorian: y{"y"}
+        final String expectFormat = "1970"; // format for skeleton y in Thai locale with Gregorian calendar
+
+        GregorianCalendar pureGregorianCalendar = new GregorianCalendar(TimeZone.GMT_ZONE, ULocale.ENGLISH);
+        pureGregorianCalendar.setGregorianChange(new Date(Long.MIN_VALUE)); // Per original bug, but not relevant
+        DateFormat df1 = DateFormat.getPatternInstance(pureGregorianCalendar, skeleton, ULocale.forLanguageTag("th"));
+        try {
+            String getFormat = df1.format(testDate);
+            if (!getFormat.equals(expectFormat)) {
+                errln("Error in DateFormat.format for th with Grego cal, expect: " + expectFormat + ", get: " + getFormat);
+            }
+        } catch (Exception e) {
+            errln("Fail in DateFormat.format for th with Grego cal: " + e);
+        }
+
+        DateFormat df2 = DateFormat.getPatternInstance(skeleton, new ULocale("th-u-ca-gregory"));
+        try {
+            String getFormat = df2.format(testDate);
+            if (!getFormat.equals(expectFormat)) {
+                errln("Error in DateFormat.format for th-u-ca-gregory, expect: " + expectFormat + ", get: " + getFormat);
+            }
+        } catch (Exception e) {
+            errln("Fail in DateFormat.format for th-u-ca-gregory: " + e);
+        }
+    }
+
     @Test
     public void TestT10110() {
         try {
