@@ -1,5 +1,5 @@
 // © 2017 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 package com.ibm.icu.dev.test.format;
 
 import java.math.BigDecimal;
@@ -9,7 +9,6 @@ import java.text.ParsePosition;
 import org.junit.Test;
 
 import com.ibm.icu.dev.test.TestUtil;
-import com.ibm.icu.dev.text.DecimalFormat_ICU58;
 import com.ibm.icu.impl.number.DecimalFormatProperties;
 import com.ibm.icu.impl.number.DecimalFormatProperties.ParseMode;
 import com.ibm.icu.impl.number.Padder.PadPosition;
@@ -290,185 +289,6 @@ public class NumberFormatDataDrivenTest {
         }
     };
 
-    /**
-     * Backwards-compatibility test: snapshot of DecimalFormat from ICU 58.
-     */
-    private DataDrivenNumberFormatTestUtility.CodeUnderTest ICU58 = new DataDrivenNumberFormatTestUtility.CodeUnderTest() {
-        @Override
-        public Character Id() {
-            return 'H';
-        }
-
-        @Override
-        public String format(DataDrivenNumberFormatTestData tuple) {
-            DecimalFormat_ICU58 fmt = createDecimalFormat(tuple);
-            String actual = fmt.format(toNumber(tuple.format));
-            String expected = tuple.output;
-            if (!expected.equals(actual)) {
-                return "Expected " + expected + ", got " + actual;
-            }
-            return null;
-        }
-
-        @Override
-        public String toPattern(DataDrivenNumberFormatTestData tuple) {
-            DecimalFormat_ICU58 fmt = createDecimalFormat(tuple);
-            StringBuilder result = new StringBuilder();
-            if (tuple.toPattern != null) {
-                String expected = tuple.toPattern;
-                String actual = fmt.toPattern();
-                if (!expected.equals(actual)) {
-                    result.append("Expected toPattern=" + expected + ", got " + actual);
-                }
-            }
-            if (tuple.toLocalizedPattern != null) {
-                String expected = tuple.toLocalizedPattern;
-                String actual = fmt.toLocalizedPattern();
-                if (!expected.equals(actual)) {
-                    result.append("Expected toLocalizedPattern=" + expected + ", got " + actual);
-                }
-            }
-            return result.length() == 0 ? null : result.toString();
-        }
-
-        @Override
-        public String parse(DataDrivenNumberFormatTestData tuple) {
-            DecimalFormat_ICU58 fmt = createDecimalFormat(tuple);
-            ParsePosition ppos = new ParsePosition(0);
-            Number actual = fmt.parse(tuple.parse, ppos);
-            return compareParseResult(tuple.output, actual, ppos);
-        }
-
-        @Override
-        public String parseCurrency(DataDrivenNumberFormatTestData tuple) {
-            DecimalFormat_ICU58 fmt = createDecimalFormat(tuple);
-            ParsePosition ppos = new ParsePosition(0);
-            CurrencyAmount actual = fmt.parseCurrency(tuple.parse, ppos);
-            return compareParseCurrencyResult(tuple.output, tuple.outputCurrency, actual, ppos);
-        }
-
-        /**
-         * @param tuple
-         * @return
-         */
-        private DecimalFormat_ICU58 createDecimalFormat(DataDrivenNumberFormatTestData tuple) {
-
-            DecimalFormat_ICU58 fmt = new DecimalFormat_ICU58(
-                    tuple.pattern == null ? "0" : tuple.pattern,
-                    new DecimalFormatSymbols(tuple.locale == null ? EN : tuple.locale));
-            adjustDecimalFormat(tuple, fmt);
-            return fmt;
-        }
-
-        /**
-         * @param tuple
-         * @param fmt
-         */
-        private void adjustDecimalFormat(DataDrivenNumberFormatTestData tuple, DecimalFormat_ICU58 fmt) {
-            if (tuple.minIntegerDigits != null) {
-                fmt.setMinimumIntegerDigits(tuple.minIntegerDigits);
-            }
-            if (tuple.maxIntegerDigits != null) {
-                fmt.setMaximumIntegerDigits(tuple.maxIntegerDigits);
-            }
-            if (tuple.minFractionDigits != null) {
-                fmt.setMinimumFractionDigits(tuple.minFractionDigits);
-            }
-            if (tuple.maxFractionDigits != null) {
-                fmt.setMaximumFractionDigits(tuple.maxFractionDigits);
-            }
-            if (tuple.currency != null) {
-                fmt.setCurrency(tuple.currency);
-            }
-            if (tuple.minGroupingDigits != null) {
-                // Oops we don't support this.
-            }
-            if (tuple.useSigDigits != null) {
-                fmt.setSignificantDigitsUsed(tuple.useSigDigits != 0);
-            }
-            if (tuple.minSigDigits != null) {
-                fmt.setMinimumSignificantDigits(tuple.minSigDigits);
-            }
-            if (tuple.maxSigDigits != null) {
-                fmt.setMaximumSignificantDigits(tuple.maxSigDigits);
-            }
-            if (tuple.useGrouping != null) {
-                fmt.setGroupingUsed(tuple.useGrouping != 0);
-            }
-            if (tuple.multiplier != null) {
-                fmt.setMultiplier(tuple.multiplier);
-            }
-            if (tuple.roundingIncrement != null) {
-                fmt.setRoundingIncrement(tuple.roundingIncrement.doubleValue());
-            }
-            if (tuple.formatWidth != null) {
-                fmt.setFormatWidth(tuple.formatWidth);
-            }
-            if (tuple.padCharacter != null && tuple.padCharacter.length() > 0) {
-                fmt.setPadCharacter(tuple.padCharacter.charAt(0));
-            }
-            if (tuple.useScientific != null) {
-                fmt.setScientificNotation(tuple.useScientific != 0);
-            }
-            if (tuple.grouping != null) {
-                fmt.setGroupingSize(tuple.grouping);
-            }
-            if (tuple.grouping2 != null) {
-                fmt.setSecondaryGroupingSize(tuple.grouping2);
-            }
-            if (tuple.roundingMode != null) {
-                fmt.setRoundingMode(tuple.roundingMode);
-            }
-            if (tuple.currencyUsage != null) {
-                fmt.setCurrencyUsage(tuple.currencyUsage);
-            }
-            if (tuple.minimumExponentDigits != null) {
-                fmt.setMinimumExponentDigits(tuple.minimumExponentDigits.byteValue());
-            }
-            if (tuple.exponentSignAlwaysShown != null) {
-                fmt.setExponentSignAlwaysShown(tuple.exponentSignAlwaysShown != 0);
-            }
-            if (tuple.decimalSeparatorAlwaysShown != null) {
-                fmt.setDecimalSeparatorAlwaysShown(tuple.decimalSeparatorAlwaysShown != 0);
-            }
-            if (tuple.padPosition != null) {
-                fmt.setPadPosition(tuple.padPosition);
-            }
-            if (tuple.positivePrefix != null) {
-                fmt.setPositivePrefix(tuple.positivePrefix);
-            }
-            if (tuple.positiveSuffix != null) {
-                fmt.setPositiveSuffix(tuple.positiveSuffix);
-            }
-            if (tuple.negativePrefix != null) {
-                fmt.setNegativePrefix(tuple.negativePrefix);
-            }
-            if (tuple.negativeSuffix != null) {
-                fmt.setNegativeSuffix(tuple.negativeSuffix);
-            }
-            if (tuple.signAlwaysShown != null) {
-                // Not supported.
-            }
-            if (tuple.localizedPattern != null) {
-                fmt.applyLocalizedPattern(tuple.localizedPattern);
-            }
-            int lenient = tuple.lenient == null ? 1 : tuple.lenient.intValue();
-            fmt.setParseStrict(lenient == 0);
-            if (tuple.parseIntegerOnly != null) {
-                fmt.setParseIntegerOnly(tuple.parseIntegerOnly != 0);
-            }
-            if (tuple.parseCaseSensitive != null) {
-                // Not supported.
-            }
-            if (tuple.decimalPatternMatchRequired != null) {
-                fmt.setDecimalPatternMatchRequired(tuple.decimalPatternMatchRequired != 0);
-            }
-            if (tuple.parseNoExponent != null) {
-                // Oops, not supported for now
-            }
-        }
-    };
-
     /**
      * Test of available JDK APIs.
      */
@@ -893,16 +713,6 @@ public class NumberFormatDataDrivenTest {
                 .runFormatSuiteIncludingKnownFailures("numberformattestspecification.txt", ICU4J);
     }
 
-    @Test
-    public void TestDataDrivenICU58() {
-        // Android can't access DecimalFormat_ICU58 for testing (ticket #13283).
-        if (TestUtil.getJavaVendor() == TestUtil.JavaVendor.Android)
-            return;
-
-        DataDrivenNumberFormatTestUtility
-                .runFormatSuiteIncludingKnownFailures("numberformattestspecification.txt", ICU58);
-    }
-
     @Test
     public void TestDataDrivenJDK() {
         // #13373: Since not all JDK implementations are the same, test only whitelisted JDKs
