@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /**
 *******************************************************************************
 * Copyright (C) 1996-2016, International Business Machines Corporation and
@@ -218,13 +218,13 @@ public class TestUScript extends TestFmwk {
             UnicodeSet scriptSet = new UnicodeSet();
             scriptSet.applyIntPropertyValue(UProperty.SCRIPT, sc);
             if(usage == ScriptUsage.NOT_ENCODED) {
-                assertTrue(sn + " not encoded, no sample", sample.length() == 0);  // Java 6: sample.isEmpty()
+                assertTrue(sn + " not encoded, no sample", sample.isEmpty());
                 assertFalse(sn + " not encoded, not RTL", UScript.isRightToLeft(sc));
                 assertFalse(sn + " not encoded, not LB letters", UScript.breaksBetweenLetters(sc));
                 assertFalse(sn + " not encoded, not cased", UScript.isCased(sc));
                 assertTrue(sn + " not encoded, no characters", scriptSet.isEmpty());
             } else {
-                assertFalse(sn + " encoded, has a sample character", sample.length() == 0);  // Java 6: sample.isEmpty()
+                assertFalse(sn + " encoded, has a sample character", sample.isEmpty());
                 int firstChar = sample.codePointAt(0);
                 int charScript = getCharScript(sc);
                 assertEquals(sn + " script(sample(script))",
@@ -329,6 +329,10 @@ public class TestUScript extends TestFmwk {
             // new in ICU 61
             "Dogra", "Gunjala_Gondi", "Makasar", "Medefaidrin",
             "Hanifi_Rohingya", "Sogdian", "Old_Sogdian",
+            // new in ICU 64
+            "Elymaic", "Nyiakeng_Puachue_Hmong", "Nandinagari", "Wancho",
+            // new in ICU 66
+            "Chorasmian", "Dives_Akuru", "Khitan_Small_Script", "Yezidi",
         };
         String[] expectedShort = new String[]{
             "Bali", "Batk", "Blis", "Brah", "Cham", "Cirt", "Cyrs", "Egyd", "Egyh", "Egyp",
@@ -361,6 +365,10 @@ public class TestUScript extends TestFmwk {
             "Gonm", "Soyo", "Zanb",
             // new in ICU 61
             "Dogr", "Gong", "Maka", "Medf", "Rohg", "Sogd", "Sogo",
+            // new in ICU 64
+            "Elym", "Hmnp", "Nand", "Wcho",
+            // new in ICU 66
+            "Chrs", "Diak", "Kits", "Yezi",
         };
         if(expectedLong.length!=(UScript.CODE_LIMIT-UScript.BALINESE)) {
             errln("need to add new script codes in lang.TestUScript.java!");
