@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /**
 *******************************************************************************
 * Copyright (C) 1996-2014, International Business Machines Corporation and
@@ -426,6 +426,19 @@ public final class UCharacterCaseTest extends TestFmwk
         }
     }
 
+    @Test
+    public void TestLithuanianTitle() {
+        ULocale LOC_LITHUANIAN = new ULocale("lt");
+
+        assertEquals("Lithuanian titlecase check in Lithuanian",
+                "\u0058\u0069\u0307\u0308",
+                UCharacter.toTitleCase(LOC_LITHUANIAN, "\u0058\u0049\u0308", null));
+
+        assertEquals("Lithuanian titlecase check in Lithuanian",
+                "\u0058\u0069\u0307\u0308",
+                UCharacter.toTitleCase(LITHUANIAN_LOCALE_, "\u0058\u0049\u0308", null));
+    }
+
     @Test
     public void TestDutchTitle() {
         ULocale LOC_DUTCH = new ULocale("nl");
@@ -791,6 +804,26 @@ public final class UCharacterCaseTest extends TestFmwk
         assertGreekUpper("ή.", "Ή.");
     }
 
+    @Test
+    public void TestArmenian() {
+        Locale hy = new Locale("hy");  // Eastern Armenian
+        Locale hyw = new Locale("hyw");  // Western Armenian
+        Locale root = Locale.ROOT;
+        // See ICU-13416:
+        // և ligature ech-yiwn
+        // uppercases to ԵՒ=ech+yiwn by default and in Western Armenian,
+        // but to ԵՎ=ech+vew in Eastern Armenian.
+        String s = "և Երևանի";
+
+        assertEquals("upper root", "ԵՒ ԵՐԵՒԱՆԻ", CaseMap.toUpper().apply(root, s));
+        assertEquals("upper hy", "ԵՎ ԵՐԵՎԱՆԻ", CaseMap.toUpper().apply(hy, s));
+        assertEquals("upper hyw", "ԵՒ ԵՐԵՒԱՆԻ", CaseMap.toUpper().apply(hyw, s));
+
+        assertEquals("title root", "Եւ Երևանի", CaseMap.toTitle().apply(root, null, s));
+        assertEquals("title hy", "Եվ Երևանի", CaseMap.toTitle().apply(hy, null, s));
+        assertEquals("title hyw", "Եւ Երևանի", CaseMap.toTitle().apply(hyw, null, s));
+    }
+
     private static final class EditChange {
         private boolean change;
         private int oldLength, newLength;
@@ -960,8 +993,8 @@ public final class UCharacterCaseTest extends TestFmwk
         // Iterate once forward, once backward, to cover more runtime conditions.
         int srcLength = expSrcIndex;
         int destLength = expDestIndex;
-        List<Integer> srcIndexes = new ArrayList<Integer>();
-        List<Integer> destIndexes = new ArrayList<Integer>();
+        List<Integer> srcIndexes = new ArrayList<>();
+        List<Integer> destIndexes = new ArrayList<>();
         srcIndexes.add(-1);
         destIndexes.add(-1);
         int srcIndex = 0;
@@ -1493,6 +1526,28 @@ public final class UCharacterCaseTest extends TestFmwk
         }
     }
 
+    @Test
+    public void TestCaseMapGreekExtended() {
+        // Ticket 13851
+        String s = "\u1F80\u1F88\u1FFC";
+        String result = CaseMap.toLower().apply(Locale.ROOT,  s);
+        assertEquals("lower", "\u1F80\u1F80\u1FF3", result);
+        result = CaseMap.toTitle().apply(Locale.ROOT, null, s);
+        assertEquals("title", "\u1F88\u1F80\u1FF3", result);
+    }
+
+    @Test
+    public void TestFoldBug20316() {
+        String s = "廬ᾒ뻪ᣃइ垚Ⴡₓ렞체ꖲ갹ݖ䕷꾬쯎㊅ᦘᰄ㸜䡏遁럢豑黾奯㸀⊻줮끎蒹衤劔뽳趧熶撒쫃窩겨ཇ脌쵐嫑⟑겭㋋濜隣ᳰ봢ℼ櫩靛㉃炔鋳" +
+                "оे⳨ᦧྃ깢粣ᑤꇪ찃̹鵄ዤꛛᰙ⡝捣쯋톐蕩栭쥀뎊ᄯ৻恳〬昴껤룩列潱ᑮ煃鶖안꽊鹭宪帐❖ा쥈잔";
+        String result = CaseMap.fold().apply(s);
+        assertTrue("廬ᾒ...->廬ἢι...", result.startsWith("廬ἢι"));
+        s = "儊ẖ깸ᝓ恷ᇁ䜄쌼ꇸჃ䗑䘬䒥㈴槁蛚紆洔㖣믏亝醣黹Ά嶨䖕篕舀ꖧ₭ଯᒗ✧ԗ墖쁳㽎苊澎긁⾆⒞蠻왃囨ᡠ邏꾭⪐턣搤穳≠톲絋砖ሷ⠆" +
+                "瞏惢鵶剕듘ᅤ♟Ԡⴠ⊡鹔ጙ갑⣚堟ᣗ✸㕇絮䠎瘗⟡놥擢ꉭ佱ྪ飹痵⿑⨴츿璿僖㯷넴鋰膄釚겼ナ黪差";
+        result = CaseMap.fold().apply(s);
+        assertTrue("儊ẖ...->儊h\u0331...", result.startsWith("儊h\u0331"));
+    }
+
     // private data members - test data --------------------------------------
 
     private static final Locale TURKISH_LOCALE_ = new Locale("tr", "TR");
@@ -1737,7 +1792,7 @@ public final class UCharacterCaseTest extends TestFmwk
      */
     private String[] getUnicodeStrings(String str)
     {
-        List<String> v = new ArrayList<String>(10);
+        List<String> v = new ArrayList<>(10);
         int start = 0;
         for (int casecount = 4; casecount > 0; casecount --) {
             int end = str.indexOf("; ", start);
