@@ -1,5 +1,5 @@
 // © 2017 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 package com.ibm.icu.dev.test.number;
 
 import static org.junit.Assert.assertEquals;
@@ -7,9 +7,9 @@ import static org.junit.Assert.fail;
 
 import org.junit.Test;
 
+import com.ibm.icu.impl.FormattedStringBuilder;
 import com.ibm.icu.impl.number.AffixUtils;
 import com.ibm.icu.impl.number.AffixUtils.SymbolProvider;
-import com.ibm.icu.impl.number.NumberStringBuilder;
 import com.ibm.icu.text.UnicodeSet;
 
 public class AffixUtilsTest {
@@ -199,19 +199,19 @@ public class AffixUtilsTest {
             }
         };
 
-        NumberStringBuilder sb = new NumberStringBuilder();
+        FormattedStringBuilder sb = new FormattedStringBuilder();
         for (String[] cas : cases) {
             String input = cas[0];
             String expected = cas[1];
             sb.clear();
-            AffixUtils.unescape(input, sb, 0, provider);
+            AffixUtils.unescape(input, sb, 0, provider, null);
             assertEquals("With symbol provider on <" + input + ">", expected, sb.toString());
         }
 
         // Test insertion position
         sb.clear();
         sb.append("abcdefg", null);
-        AffixUtils.unescape("-+%", sb, 4, provider);
+        AffixUtils.unescape("-+%", sb, 4, provider, null);
         assertEquals("Symbol provider into middle", "abcd123efg", sb.toString());
     }
 
@@ -236,8 +236,8 @@ public class AffixUtilsTest {
     }
 
     private static String unescapeWithDefaults(String input) {
-        NumberStringBuilder nsb = new NumberStringBuilder();
-        int length = AffixUtils.unescape(input, nsb, 0, DEFAULT_SYMBOL_PROVIDER);
+        FormattedStringBuilder nsb = new FormattedStringBuilder();
+        int length = AffixUtils.unescape(input, nsb, 0, DEFAULT_SYMBOL_PROVIDER, null);
         assertEquals("Return value of unescape", nsb.length(), length);
         return nsb.toString();
     }
