@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 
 package com.ibm.icu.dev.test.rbbi;
 
@@ -60,10 +60,11 @@ public class RBBIMonkeyTest extends TestFmwk {
     //                    is compiled to a regular expression.
 
     static class BreakRule {
-        String    fName;                   // Name of the rule.
-        String    fRule;                   // Rule expression, excluding the name, as written in user source.
-        String    fExpandedRule;           // Rule expression after expanding the set definitions.
-        Matcher   fRuleMatcher;            // Regular expression that matches the rule.
+        String    fName;                      // Name of the rule.
+        String    fRule;                      // Rule expression, excluding the name, as written in user source.
+        String    fExpandedRule;              // Rule expression after expanding the set definitions.
+        Matcher   fRuleMatcher;               // Regular expression that matches the rule.
+        boolean   fInitialMatchOnly = false;  // True if rule begins with '^', meaning no chaining.
     };
 
 
@@ -73,10 +74,10 @@ public class RBBIMonkeyTest extends TestFmwk {
     static class BreakRules {
         BreakRules(RBBIMonkeyImpl monkeyImpl) {
             fMonkeyImpl = monkeyImpl;
-            fBreakRules = new ArrayList<BreakRule>();
+            fBreakRules = new ArrayList<>();
             fType = BreakIterator.KIND_TITLE;
-            fCharClasses = new HashMap<String, CharClass>();
-            fCharClassList = new ArrayList<CharClass>();
+            fCharClasses = new HashMap<>();
+            fCharClassList = new ArrayList<>();
             fDictionarySet = new UnicodeSet();
 
             // Match an alpha-numeric identifier in a rule. Will be a set name.
@@ -220,6 +221,14 @@ public class RBBIMonkeyTest extends TestFmwk {
             }
             fPropertyMatcher.appendTail(expandedRule);
 
+            // If rule begins with a '^' rule chaining is disallowed.
+            // Strip off the '^' from the rule expression, and set the flag.
+            if (expandedRule.charAt(0) == '^') {
+                thisRule.fInitialMatchOnly = true;
+                expandedRule.deleteCharAt(0);
+                expandedRule = new StringBuffer(expandedRule.toString().trim());
+            }
+
             //   Replace any [^negated sets] with equivalent flattened sets generated by
             //   ICU UnicodeSet. [^ ...] in Java Regex character classes does not apply
             //   to any nested classes. Variable substitution in rules produces
@@ -293,25 +302,12 @@ public class RBBIMonkeyTest extends TestFmwk {
 
             thisRule.fExpandedRule = thisRule.fExpandedRule.replace("[]", "[a&&[^a]]");
 
-            // Change Unicode escape syntax for compatibility with Java regular expressions (Java 7 or newer)
+            // Change Unicode escape syntax for compatibility with Java regular expressions
             //    \udddd     => \x{dddd}
             //    \U00hhhhhh => \x{hhhhhh}
 
-            // thisRule.fExpandedRule = thisRule.fExpandedRule.replaceAll("\\\\u([0-9A-Fa-f]{4})", "\\\\x{$1}");
-            // thisRule.fExpandedRule = thisRule.fExpandedRule.replaceAll("\\\\U00([0-9A-Fa-f]{6})", "\\\\x{$1}");
-
-            // Java 6 compatibility troubles - there is no syntax for escaping a supplementary character
-            // within a regular expression character class. Put them in as unescaped literal chars.
-            StringBuilder sb = new StringBuilder(thisRule.fExpandedRule);
-            while (true) {
-                int where = sb.indexOf("\\U00");
-                if (where < 0) {
-                    break;
-                }
-                String cp = hexToCodePoint(sb.substring(where+2, where+10));
-                sb.replace(where, where+10, cp);
-            }
-            thisRule.fExpandedRule = sb.toString();
+             thisRule.fExpandedRule = thisRule.fExpandedRule.replaceAll("\\\\u([0-9A-Fa-f]{4})", "\\\\x{$1}");
+             thisRule.fExpandedRule = thisRule.fExpandedRule.replaceAll("\\\\U00([0-9A-Fa-f]{6})", "\\\\x{$1}");
 
             // Escape any literal '#' in the rule expression. Without escaping, these introduce a comment.
             // UnicodeSet._generatePattern() inserts un-escaped "#"s
@@ -336,6 +332,7 @@ public class RBBIMonkeyTest extends TestFmwk {
             fBreakRules.add(thisRule);
         };
 
+        @SuppressWarnings("unused")
         private static String hexToCodePoint(String hex) {
             int cp = Integer.parseInt(hex, 16);
             return new StringBuilder().appendCodePoint(cp).toString();
@@ -562,6 +559,9 @@ public class RBBIMonkeyTest extends TestFmwk {
                                              // ICU always reports a break there.
                                              // The reference rules do not have a means to do so.
             int strIdx = 0;
+            boolean initialMatch = true;     // True at start of text, and immediately after each boundary,
+            //                               // for control over rule chaining.
+
             while (strIdx < fString.length()) {
                 BreakRule matchingRule = null;
                 boolean hasBreak = false;
@@ -570,6 +570,10 @@ public class RBBIMonkeyTest extends TestFmwk {
                 int matchEnd = 0;
                 for (ruleNum=0; ruleNum<rules.fBreakRules.size(); ruleNum++) {
                     BreakRule rule = rules.fBreakRules.get(ruleNum);
+                    if (rule.fInitialMatchOnly && !initialMatch) {
+                        // Skip checking this '^' rule. (No rule chaining)
+                        continue;
+                    }
                     rule.fRuleMatcher.reset(fString.substring(strIdx));
                     if (rule.fRuleMatcher.lookingAt()) {
                         // A candidate rule match, check further to see if we take it or continue to check other rules.
@@ -620,6 +624,7 @@ public class RBBIMonkeyTest extends TestFmwk {
                     // which may differ from end of the match. The matching rule may have included
                     // context following the boundary that needs to be looked at again.
                     strIdx = breakPos;
+                    initialMatch = true;
                 } else {
                     // Original rule didn't specify a break.
                     // Continue applying rules starting on the last code point of this match.
@@ -631,6 +636,7 @@ public class RBBIMonkeyTest extends TestFmwk {
                                 rules.fMonkeyImpl.fRuleFileName, matchingRule.fName));
                     }
                     strIdx = updatedStrIdx;
+                    initialMatch = false;
                 }
             }
         };
@@ -999,7 +1005,7 @@ public class RBBIMonkeyTest extends TestFmwk {
     //
     @Test
     public void TestMonkey() {
-        String tests[] = {"grapheme.txt", "word.txt", "line.txt", "sentence.txt", "line_normal.txt",
+        String tests[] = {"grapheme.txt", "word.txt", "line.txt", "line_cj.txt", "sentence.txt", "line_normal.txt",
                 "line_normal_cj.txt", "line_loose.txt", "line_loose_cj.txt", "word_POSIX.txt"
         };
 
@@ -1014,7 +1020,7 @@ public class RBBIMonkeyTest extends TestFmwk {
         boolean verbose = getBooleanProperty("verbose", false);
         int seed = getIntProperty("seed", 1);
 
-        List<RBBIMonkeyImpl> startedTests = new ArrayList<RBBIMonkeyImpl>();
+        List<RBBIMonkeyImpl> startedTests = new ArrayList<>();
 
         // Monkey testing is multi-threaded.
         // Each set of break rules to be tested is run in a separate thread.
