@@ -1,5 +1,5 @@
 // © 2016 and later: Unicode, Inc. and others.
-// License & terms of use: http://www.unicode.org/copyright.html#License
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  * Copyright (c) 2004-2014, International Business Machines
@@ -13,12 +13,12 @@ import java.text.AttributedCharacterIterator;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.Locale;
+import java.util.Objects;
 
 import com.ibm.icu.impl.DateNumberFormat;
 import com.ibm.icu.impl.TZDBTimeZoneNames;
 import com.ibm.icu.impl.TimeZoneGenericNames;
 import com.ibm.icu.impl.TimeZoneGenericNames.GenericNameType;
-import com.ibm.icu.impl.Utility;
 import com.ibm.icu.text.ChineseDateFormat;
 import com.ibm.icu.text.ChineseDateFormatSymbols;
 import com.ibm.icu.text.CompactDecimalFormat;
@@ -30,10 +30,12 @@ import com.ibm.icu.text.DateIntervalInfo;
 import com.ibm.icu.text.DecimalFormat;
 import com.ibm.icu.text.DecimalFormatSymbols;
 import com.ibm.icu.text.DurationFormat;
+import com.ibm.icu.text.ListFormatter;
 import com.ibm.icu.text.MessageFormat;
 import com.ibm.icu.text.NumberFormat;
 import com.ibm.icu.text.PluralFormat;
 import com.ibm.icu.text.PluralRules;
+import com.ibm.icu.text.RelativeDateTimeFormatter;
 import com.ibm.icu.text.RuleBasedNumberFormat;
 import com.ibm.icu.text.SelectFormat;
 import com.ibm.icu.text.SimpleDateFormat;
@@ -1106,8 +1108,10 @@ public class FormatHandler
             NumberFormat format_b = (NumberFormat) b;
             double number = 1234.56;
 
-            String result_a = format_a.format(number);
-            String result_b = format_b.format(number);
+            // In CLDR 34, French changed grouping separator from \u00a0 to \u202f;
+            // here we undo that in formatted result so serialization tests don't fail
+            String result_a = format_a.format(number).replace('\u202f', '\u00a0');
+            String result_b = format_b.format(number).replace('\u202f', '\u00a0');
             boolean equal = result_a.equals(result_b);
             if (!equal) {
                 System.out.println(format_a+" "+format_b);
@@ -1798,6 +1802,66 @@ public class FormatHandler
         }
     }
 
+    public static class RelativeDateTimeFormatterFieldHandler implements SerializableTestUtility.Handler
+    {
+        @Override
+        public Object[] getTestObjects()
+        {
+            return new Object[] {RelativeDateTimeFormatter.Field.LITERAL};
+        }
+
+        @Override
+        public boolean hasSameBehavior(Object a, Object b)
+        {
+            return (a == b);
+        }
+    }
+
+    public static class DateIntervalSpanFieldHandler implements SerializableTestUtility.Handler
+    {
+        @Override
+        public Object[] getTestObjects()
+        {
+            return new Object[] {DateIntervalFormat.SpanField.DATE_INTERVAL_SPAN};
+        }
+
+        @Override
+        public boolean hasSameBehavior(Object a, Object b)
+        {
+            return (a == b);
+        }
+    }
+
+    public static class ListFormatterFieldHandler implements SerializableTestUtility.Handler
+    {
+        @Override
+        public Object[] getTestObjects()
+        {
+            return new Object[] {ListFormatter.Field.ELEMENT, ListFormatter.Field.LITERAL};
+        }
+
+        @Override
+        public boolean hasSameBehavior(Object a, Object b)
+        {
+            return (a == b);
+        }
+    }
+
+    public static class ListFormatterSpanFieldHandler implements SerializableTestUtility.Handler
+    {
+        @Override
+        public Object[] getTestObjects()
+        {
+            return new Object[] {ListFormatter.SpanField.LIST_SPAN};
+        }
+
+        @Override
+        public boolean hasSameBehavior(Object a, Object b)
+        {
+            return (a == b);
+        }
+    }
+
     public static class DateFormatHandler implements SerializableTestUtility.Handler
     {
         static HashMap cannedPatterns = new HashMap();
@@ -2378,7 +2442,7 @@ public class FormatHandler
                     for (long date : DATES) {
                         String nameA = tzgna.getDisplayName(tz, nt, date);
                         String nameB = tzgnb.getDisplayName(tz, nt, date);
-                        if (!Utility.objectEquals(nameA, nameB)) {
+                        if (!Objects.equals(nameA, nameB)) {
                             return false;
                         }
                     }
@@ -2424,7 +2488,7 @@ public class FormatHandler
                     for (long date : DATES) {
                         String nameA = tzdbna.getDisplayName(tzid, nt, date);
                         String nameB = tzdbnb.getDisplayName(tzid, nt, date);
-                        if (!Utility.objectEquals(nameA, nameB)) {
+                        if (!Objects.equals(nameA, nameB)) {
                             return false;
                         }
                     }
